<?php
/*
 *  $Id: Default.php 402 2011-11-14 23:08:31Z k42b3.x@googlemail.com $
 *
 * amun
 * A social content managment system based on the psx framework. For
 * the current version and informations visit <http://amun.phpsx.org>
 *
 * Copyright (c) 2010 Christoph Kappestein <k42b3.x@gmail.com>
 *
 * This file is part of amun. amun is free software: you can
 * redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or any later version.
 *
 * amun is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with amun. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * Amun_Base_Default
 *
 * @author     Christoph Kappestein <k42b3.x@gmail.com>
 * @license    http://www.gnu.org/licenses/gpl.html GPLv3
 * @link       http://amun.phpsx.org
 * @category   Amun
 * @package    Amun_Base
 * @version    $Revision: 402 $
 */
class Amun_Base_Default extends ArrayObject
{
	protected static $_instance;

	protected $container = array();

	protected $config;
	protected $sql;
	protected $registry;
	protected $tableRegistry;
	protected $validate;
	protected $get;
	protected $post;

	public function __construct(PSX_Config $config)
	{
		parent::__construct($this->container, parent::ARRAY_AS_PROPS);


		// config
		$this->config = $config;

		$this->offsetSet('PSX_Config', $this->config);


		// sql
		$this->sql = new PSX_Sql(new PSX_Sql_Driver_Pdo(), $config['psx_sql_host'], $config['psx_sql_user'], $config['psx_sql_pw'], $config['psx_sql_db']);

		$this->offsetSet('PSX_Sql', $this->sql);


		// registry
		$this->registry = Amun_Registry::initInstance($this->config, $this->sql);

		$this->offsetSet('Amun_Registry', $this->registry);


		// table registry
		$this->tableRegistry = Amun_Sql_Table_Registry::initInstance($this->registry);

		$this->offsetSet('Amun_Sql_Table_Registry', $this->tableRegistry);


		// validate
		$this->validate = new PSX_Validate();

		$this->offsetSet('PSX_Validate', $this->validate);


		// get
		$this->get = new PSX_Get($this->validate);

		$this->offsetSet('PSX_Get', $this->get);


		// post
		$this->post = new PSX_Post($this->validate);

		$this->offsetSet('PSX_Post', $this->post);
	}

	public function getConfig()
	{
		return $this->config;
	}

	public function getSql()
	{
		return $this->sql;
	}

	public function getRegistry()
	{
		return $this->registry;
	}

	public function getTableRegistry()
	{
		return $this->tableRegistry;
	}

	public function getValidate()
	{
		return $this->validate;
	}

	public function getGet()
	{
		return $this->get;
	}

	public function getPost()
	{
		return $this->post;
	}

	public static function initInstance(PSX_Config $config)
	{
		return self::$_instance = new self($config);
	}

	public static function getInstance()
	{
		if(self::$_instance === null)
		{
			self::$_instance = new self();
		}

		return self::$_instance;
	}

	public static function get($key)
	{
		return self::getInstance()->offsetGet($key);
	}

	public static function set($key, $value)
	{
		self::getInstance()->offsetSet($key, $value);
	}
}

