<?php
/*
 *  $Id: Form.php 405 2011-11-16 21:42:16Z k42b3.x@googlemail.com $
 *
 * amun
 * A social content managment system based on the psx framework. For
 * the current version and informations visit <http://amun.phpsx.org>
 *
 * Copyright (c) 2010 Christoph Kappestein <k42b3.x@gmail.com>
 *
 * This file is part of amun. amun is free software: you can
 * redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or any later version.
 *
 * amun is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with amun. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * Amun_Content_Gadget_Form
 *
 * @author     Christoph Kappestein <k42b3.x@gmail.com>
 * @license    http://www.gnu.org/licenses/gpl.html GPLv3
 * @link       http://amun.phpsx.org
 * @category   Amun
 * @package    Amun_Content_Gadget
 * @version    $Revision: 405 $
 */
class Amun_Content_Gadget_Form extends Amun_Data_FormAbstract
{
	public function create()
	{
		$form = new Amun_Form('POST', $this->url);


		$panel = new Amun_Form_Element_Panel('gadget', 'Gadget');


		$title = new Amun_Form_Element_Input('title', 'Title');
		$title->setType('text');

		$panel->add($title);


		$path = new Amun_Form_Element_Select('path', 'Path');
		$path->setOptions($this->listGadget());

		$panel->add($path);


		$cache = new Amun_Form_Element_Input('cache', 'Cache');
		$cache->setType('checkbox');

		$panel->add($cache);


		$expire = new Amun_Form_Element_Input('expire', 'Expire');
		$expire->setType('text');

		$panel->add($expire);


		if($this->user->isAnonymous())
		{
			$captcha = new Amun_Form_Element_Captcha('captcha', 'Captcha');
			$captcha->setSrc($this->config['psx_url'] . '/' . $this->config['psx_dispatch'] . 'api/system/captcha');

			$panel->add($captcha);
		}


		$form->setContainer($panel);


		return $form;
	}

	public function update($id)
	{
		$record = Amun_Sql_Table_Registry::get('Content_Gadget')->getRecord($id);


		$form = new Amun_Form('PUT', $this->url);


		$panel = new Amun_Form_Element_Panel('gadget', 'Gadget');


		$id = new Amun_Form_Element_Input('id', 'Id', $record->id);
		$id->setType('hidden');

		$panel->add($id);


		$title = new Amun_Form_Element_Input('title', 'Title', $record->title);
		$title->setType('text');

		$panel->add($title);


		$path = new Amun_Form_Element_Select('path', 'Path', $record->path);
		$path->setOptions($this->listGadget());

		$panel->add($path);


		$cache = new Amun_Form_Element_Input('cache', 'Cache', $record->cache);
		$cache->setType('checkbox');

		$panel->add($cache);


		$expire = new Amun_Form_Element_Input('expire', 'Expire', $record->expire);
		$expire->setType('text');

		$panel->add($expire);


		if($this->user->isAnonymous())
		{
			$captcha = new Amun_Form_Element_Captcha('captcha', 'Captcha');
			$captcha->setSrc($this->config['psx_url'] . '/' . $this->config['psx_dispatch'] . 'api/system/captcha');

			$panel->add($captcha);
		}


		$form->setContainer($panel);


		return $form;
	}

	public function delete($id)
	{
		$record = Amun_Sql_Table_Registry::get('Content_Gadget')->getRecord($id);


		$form = new Amun_Form('DELETE', $this->url);


		$panel = new Amun_Form_Element_Panel('gadget', 'Gadget');


		$id = new Amun_Form_Element_Input('id', 'Id', $record->id);
		$id->setType('hidden');

		$panel->add($id);


		$title = new Amun_Form_Element_Input('title', 'Title', $record->title);
		$title->setType('text');
		$title->setDisabled(true);

		$panel->add($title);


		$path = new Amun_Form_Element_Select('path', 'Path', $record->path);
		$path->setOptions($this->listGadget());
		$path->setDisabled(true);

		$panel->add($path);


		$cache = new Amun_Form_Element_Input('cache', 'Cache', $record->cache);
		$cache->setType('checkbox');
		$cache->setDisabled(true);

		$panel->add($cache);


		$expire = new Amun_Form_Element_Input('expire', 'Expire', $record->expire);
		$expire->setType('text');
		$expire->setDisabled(true);

		$panel->add($expire);


		if($this->user->isAnonymous())
		{
			$captcha = new Amun_Form_Element_Captcha('captcha', 'Captcha');
			$captcha->setSrc($this->config['psx_url'] . '/' . $this->config['psx_dispatch'] . 'api/system/captcha');

			$panel->add($captcha);
		}


		$form->setContainer($panel);


		return $form;
	}

	private function listGadget()
	{
		$path = PSX_PATH_MODULE . '/gadget';

		$data = $this->scanDir($path);

		return $data;
	}

	private function scanDir($path, $base = NULL)
	{
		$list = array();

		if($base === NULL)
		{
			$base = $path;
		}

		if(is_dir($path))
		{
			$dirs = scandir($path);

			foreach($dirs as $d)
			{
				if($d[0] != '.')
				{
					$item = $path . '/' . $d;
					$ext  = pathinfo($item, PATHINFO_EXTENSION);

					if(is_dir($item))
					{
						$list = array_merge($list, $this->scanDir($item, $path));
					}

					if(is_file($item) && $ext == 'php')
					{
						$rel_path = substr($item, strlen($base) + 1);

						array_push($list, array(

							'label' => $rel_path,
							'value' => $rel_path,

						));
					}
				}
			}
		}

		return $list;
	}
}

