<?php
/*
 *  $Id: Media.php 416 2011-11-19 22:01:44Z k42b3.x@googlemail.com $
 *
 * amun
 * A social content managment system based on the psx framework. For
 * the current version and informations visit <http://amun.phpsx.org>
 *
 * Copyright (c) 2010 Christoph Kappestein <k42b3.x@gmail.com>
 *
 * This file is part of amun. amun is free software: you can
 * redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or any later version.
 *
 * amun is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with amun. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * Amun_Content_Media
 *
 * @author     Christoph Kappestein <k42b3.x@gmail.com>
 * @license    http://www.gnu.org/licenses/gpl.html GPLv3
 * @link       http://amun.phpsx.org
 * @category   Amun
 * @package    Amun_Content_Media
 * @version    $Revision: 416 $
 */
class Amun_Content_Media extends Amun_Data_RecordAbstract
{
	protected $_user;

	public function getName()
	{
		return 'media';
	}

	public function setId($id)
	{
		$id = $this->_validate->apply($id, 'integer', array(new Amun_Filter_Id($this->_table)), 'id', 'Id');

		if(!$this->_validate->hasError())
		{
			$this->id = $id;
		}
		else
		{
			throw new PSX_Data_Exception($this->_validate->getLastError());
		}
	}

	public function setTitle($title)
	{
		$title = $this->_validate->apply($title, 'string', array(new PSX_Filter_Length(3, 128), new PSX_Filter_Html()), 'title', 'Title');

		if(!$this->_validate->hasError())
		{
			$this->title = $title;
		}
		else
		{
			throw new PSX_Data_Exception($this->_validate->getLastError());
		}
	}

	public function setPath($path)
	{
		$path = $this->_validate->apply($path, 'string', array(new Amun_Content_Media_Filter_Path($this->_config)), 'path', 'Path');

		if(!$this->_validate->hasError())
		{
			$this->path = $path;
		}
		else
		{
			throw new PSX_Data_Exception($this->_validate->getLastError());
		}
	}

	public function setType($type)
	{
		$type = $this->_validate->apply($type, 'string', array(new PSX_Filter_KeyExists(self::getType())), 'type', 'Type');

		if(!$this->_validate->hasError())
		{
			$this->type = $type;
		}
		else
		{
			throw new PSX_Data_Exception($this->_validate->getLastError());
		}
	}

	public function setSize($size)
	{
		$this->size = intval($size);
	}

	public function setMimeType($mimeType)
	{
		$mimeType = $this->_validate->apply($mimeType, 'string', array(new PSX_Filter_KeyExists(self::getMimeType())), 'mimeType', 'MIME Type');

		if(!$this->_validate->hasError())
		{
			$this->mimeType = $mimeType;
		}
		else
		{
			throw new PSX_Data_Exception($this->_validate->getLastError());
		}
	}

	public function getId()
	{
		return $this->_config->getUrn('content', 'media', $this->id);
	}

	public function getUser()
	{
		if($this->_user === null)
		{
			$this->_user = Amun_Sql_Table_Registry::get('User_Account')->getRecord($this->userId);
		}

		return $this->_user;
	}

	public function getDate()
	{
		if($this->_date === null)
		{
			$this->_date = new DateTime($this->date, $this->_registry['core.default_timezone']);
		}

		return $this->_date;
	}

	public static function getType($type = false)
	{
		$t = array(

			'application' => 'Application',
			'audio'       => 'Audio',
			'text'        => 'Text',
			'image'       => 'Image',
			'video'       => 'Video',

		);

		if($type !== false)
		{
			if(array_key_exists($type, $t))
			{
				return $t[$type];
			}
			else
			{
				return false;
			}
		}
		else
		{
			return $t;
		}
	}

	public static function getMimeType($mime_type = false)
	{
		$m = array(

			'application/json'              => 'JSON',
			'application/octet-stream'      => 'Binary',
			'application/pdf'               => 'Adobe Acrobat',
			'application/xml'               => 'XML',
			'application/x-javascript'      => 'JavaScript',
			'application/x-shockwave-flash' => 'Macromedia Shockwave',
			'audio/midi'                    => 'MIDI',
			'audio/mpeg'                    => 'MPEG audio',
			'audio/ogg'                     => 'OGG audio',
			'audio/x-aiff'                  => 'AIFF audio',
			'audio/x-mpeg'                  => 'MPEG audio',
			'audio/x-wav'                   => 'Microsoft Windows WAVE audio',
			'image/bmp'                     => 'Bitmap',
			'image/gif'                     => 'GIF image',
			'image/jpeg'                    => 'JPEG image',
			'image/png'                     => 'PNG image',
			'image/tiff'                    => 'TIFF image',
			'text/html'                     => 'HTML',
			'text/plain'                    => 'Plain text',
			'video/mpeg'                    => 'MPEG video',
			'video/msvideo'                 => 'Microsoft Windows video',
			'video/ogg'                     => 'OGG video',
			'video/quicktime'               => 'QuickTime video',

		);

		if($mime_type !== false)
		{
			if(array_key_exists($mime_type, $m))
			{
				return $m[$mime_type];
			}
			else
			{
				return false;
			}
		}
		else
		{
			return $m;
		}
	}
}


