<?php
/*
 *  $Id: Handler.php 404 2011-11-16 20:42:53Z k42b3.x@googlemail.com $
 *
 * amun
 * A social content managment system based on the psx framework. For
 * the current version and informations visit <http://amun.phpsx.org>
 *
 * Copyright (c) 2010 Christoph Kappestein <k42b3.x@gmail.com>
 *
 * This file is part of amun. amun is free software: you can
 * redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or any later version.
 *
 * amun is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with amun. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * Amun_Content_Media_Handler
 *
 * @author     Christoph Kappestein <k42b3.x@gmail.com>
 * @license    http://www.gnu.org/licenses/gpl.html GPLv3
 * @link       http://amun.phpsx.org
 * @category   Amun
 * @package    Amun_Content_Media
 * @version    $Revision: 404 $
 */
class Amun_Content_Media_Handler extends Amun_Data_HandlerAbstract
{
	public function create(PSX_Data_RecordInterface $record)
	{
		throw new PSX_Data_Exception('Create a media record is not possible');
	}

	public function update(PSX_Data_RecordInterface $record)
	{
		throw new PSX_Data_Exception('Update a media record is not possible');
	}

	public function delete(PSX_Data_RecordInterface $record)
	{
		if($record->hasFields('id'))
		{
			$con = new PSX_Sql_Condition(array('id', '=', $record['id']));


			// delete media file
			$path     = $this->sql->select($this->registry['table.content_media'], array('path'), $con, PSX_Sql::SELECT_FIELD);

			$realPath = $this->registry['core.media_path'] . '/' . $this->user->name . '/' . $path;

			if(is_file($realPath))
			{
				unlink($realPath);
			}


			$this->table->delete($con);


			$this->notify(Amun_Data_RecordAbstract::DELETE, $record);


			return $record;
		}
		else
		{
			throw new PSX_Data_Exception('Missing field in record');
		}
	}

	public function upload(array $file)
	{
		if($this->isValidUpload($file))
		{
			// check max upload size for user
			$sum = $this->sql->getField('SELECT SUM(size) FROM `' . $this->getTable() . '` WHERE userId = ' . $this->user->id . ' GROUP BY userId');

			if($sum > $this->registry['core.media_upload_size'])
			{
				throw new PSX_Data_Exception('Max upload size reached per user');
			}


			// check user dir
			$folder = $this->registry['core.media_path'] . '/' . $this->user->name;

			if(!is_dir($folder))
			{
				if(!mkdir($folder))
				{
					throw new PSX_Data_Exception('Could not create user dir');
				}
			}


			$path     = sha1($file['name'] . time() . uniqid(mt_rand(), true)) . '.tmp';
			$realPath = $folder . '/' . $path;

			if(move_uploaded_file($file['tmp_name'], $realPath))
			{
				try
				{
					$mimeType = $file['type'];
					$type     = $this->getType($mimeType);
					$size     = filesize($realPath);

					if($type === false)
					{
						throw new PSX_Data_Exception('Invalide type');
					}


					// create record
					$date = new DateTime('NOW', $this->registry['core.default_timezone']);

					$record = Amun_Sql_Table_Registry::get('Content_Media')->getRecord();
					$record->setTitle($file['name']);
					$record->setPath($path);
					$record->setType($type);
					$record->setSize($size);
					$record->setMimeType($mimeType);

					$record->userId = $this->user->id;
					$record->date = $date->format(PSX_Time::SQL);


					$this->table->insert($record->getFields());


					$record->id = $this->sql->getLastInsertId();

					$this->notify(Amun_Data_RecordAbstract::INSERT, $record);


					return true;
				}
				catch(Exception $e)
				{
					unlink($realPath);

					throw $e;
				}
			}
			else
			{
				throw new PSX_Data_Exception('Couldnt move uploaded file');
			}
		}
		else
		{
			throw new PSX_Data_Exception('No file was uploaded');
		}
	}

	private function isValidUpload(array $file)
	{
		if(!$this->registry['core.media_upload'])
		{
			throw new PSX_Data_Exception('Upload ist not enabled');
		}

		if(!(isset($file['name']) && isset($file['type']) && isset($file['size']) && isset($file['tmp_name']) && isset($file['error'])))
		{
			throw new PSX_Data_Exception('Invalid file');
		}

		$headers = PSX_Config::getRequestHeader();

		if(strpos($headers['content-type'], 'multipart/form-data') !== false)
		{
			switch($file['error'])
			{
				case UPLOAD_ERR_OK:

					return is_uploaded_file($file['tmp_name']);

					break;

				case UPLOAD_ERR_INI_SIZE:

					throw new PSX_Data_Exception('The uploaded file exceeds the upload_max_filesize directive in php.ini');

					break;

				case UPLOAD_ERR_FORM_SIZE:

					throw new PSX_Data_Exception('The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form');

					break;

				case UPLOAD_ERR_PARTIAL:

					throw new PSX_Data_Exception('The uploaded file was only partially uploaded');

					break;

				case UPLOAD_ERR_NO_FILE:

					throw new PSX_Data_Exception('No file was uploaded');

					break;

				case UPLOAD_ERR_NO_TMP_DIR:

					throw new PSX_Data_Exception('Missing a temporary folder');

					break;

				case UPLOAD_ERR_CANT_WRITE:

					throw new PSX_Data_Exception('Failed to write file to disk');

					break;

				case UPLOAD_ERR_EXTENSION:

					throw new PSX_Data_Exception('A PHP extension stopped the file upload');

					break;
			}
		}
		else
		{
			throw new PSX_Data_Exception('Invalid content type');
		}
	}

	private function getType($mimeType)
	{
		$mimeType = strtolower($mimeType);
		$types    = Amun_Content_Media::getType();

		foreach($types as $type => $val)
		{
			if(strpos($mimeType, $type) !== false)
			{
				return $type;
			}
		}

		return false;
	}
}

