<?php
/*
 *  $Id: Form.php 405 2011-11-16 21:42:16Z k42b3.x@googlemail.com $
 *
 * amun
 * A social content managment system based on the psx framework. For
 * the current version and informations visit <http://amun.phpsx.org>
 *
 * Copyright (c) 2010 Christoph Kappestein <k42b3.x@gmail.com>
 *
 * This file is part of amun. amun is free software: you can
 * redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or any later version.
 *
 * amun is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with amun. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * Amun_Content_Service_Form
 *
 * @author     Christoph Kappestein <k42b3.x@gmail.com>
 * @license    http://www.gnu.org/licenses/gpl.html GPLv3
 * @link       http://amun.phpsx.org
 * @category   Amun
 * @package    Amun_Content_Service
 * @version    $Revision: 405 $
 */
class Amun_Content_Service_Form extends Amun_Data_FormAbstract
{
	public function create()
	{
		$form = new Amun_Form('POST', $this->url);


		$panel = new Amun_Form_Element_Panel('service', 'Service');


		$path = new Amun_Form_Element_Select('name', 'Name');
		$path->setOptions($this->getInstallableService());

		$panel->add($path);


		if($this->user->isAnonymous())
		{
			$captcha = new Amun_Form_Element_Captcha('captcha', 'Captcha');
			$captcha->setSrc($this->config['psx_url'] . '/' . $this->config['psx_dispatch'] . 'api/system/captcha');

			$panel->add($captcha);
		}


		$form->setContainer($panel);


		return $form;
	}

	public function update($id)
	{
		throw new PSX_Data_Exception('Update a service record is not possible');
	}

	public function delete($id)
	{
		$record = Amun_Sql_Table_Registry::get('Content_Service')->getRecord($id);


		$form = new Amun_Form('DELETE', $this->url);


		$panel = new Amun_Form_Element_Panel('service', 'Service');


		$id = new Amun_Form_Element_Input('id', 'ID', $record->id);
		$id->setType('hidden');

		$panel->add($id);


		$name = new Amun_Form_Element_Input('name', 'Name', $record->name);
		$name->setType('text');
		$name->setDisabled(true);

		$panel->add($name);


		$type = new Amun_Form_Element_Input('type', 'Type', $record->type);
		$type->setType('text');
		$type->setDisabled(true);

		$panel->add($type);


		$link = new Amun_Form_Element_Input('link', 'Link', $record->link);
		$link->setType('text');
		$link->setDisabled(true);

		$panel->add($link);


		$author = new Amun_Form_Element_Input('author', 'Author', $record->author);
		$author->setType('text');
		$author->setDisabled(true);

		$panel->add($author);


		$license = new Amun_Form_Element_Input('license', 'License', $record->license);
		$license->setType('text');
		$license->setDisabled(true);

		$panel->add($license);


		$version = new Amun_Form_Element_Input('version', 'Version', $record->version);
		$version->setType('text');
		$version->setDisabled(true);

		$panel->add($version);


		if($this->user->isAnonymous())
		{
			$captcha = new Amun_Form_Element_Captcha('captcha', 'Captcha');
			$captcha->setSrc($this->config['psx_url'] . '/' . $this->config['psx_dispatch'] . 'api/system/captcha');

			$panel->add($captcha);
		}


		$form->setContainer($panel);


		return $form;
	}

	private function getInstallableService()
	{
		$services = array();
		$installedServices = $this->sql->getCol('SELECT name FROM ' . $this->registry['table.content_service']);

		$path = $this->config['amun_service_path'];
		$dirs = scandir($path);

		foreach($dirs as $f)
		{
			if($f != '.' && $f != '..')
			{
				$item = $path . '/' . $f;

				if(is_dir($item))
				{
					$config = $item . '/config.xml';

					if(is_file($config))
					{
						$xml = simplexml_load_file($config);

						if(isset($xml->name))
						{
							$name = strval($xml->name);

							if(!in_array($name, $installedServices))
							{
								array_push($services, array(

									'label' => ucfirst($name),
									'value' => $f,

								));
							}
						}
					}
				}
			}
		}

		return $services;
	}
}

