<?php
/*
 *  $Id: Handler.php 423 2011-11-28 22:09:05Z k42b3.x@googlemail.com $
 *
 * amun
 * A social content managment system based on the psx framework. For
 * the current version and informations visit <http://amun.phpsx.org>
 *
 * Copyright (c) 2010 Christoph Kappestein <k42b3.x@gmail.com>
 *
 * This file is part of amun. amun is free software: you can
 * redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or any later version.
 *
 * amun is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with amun. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * Amun_Content_Service_Handler
 *
 * @author     Christoph Kappestein <k42b3.x@gmail.com>
 * @license    http://www.gnu.org/licenses/gpl.html GPLv3
 * @link       http://amun.phpsx.org
 * @category   Amun
 * @package    Amun_Content_Service
 * @version    $Revision: 423 $
 */
class Amun_Content_Service_Handler extends Amun_Data_HandlerAbstract
{
	private $ids = array();
	private $serviceId;

	public function create(PSX_Data_RecordInterface $record)
	{
		if($record->hasFields('name'))
		{
			// already installed
			$con = new PSX_Sql_Condition(array('name', '=', $record->name));

			if($this->table->count($con) > 0)
			{
				throw new PSX_Data_Exception('Service already installed');
			}


			// parse config
			$config = $this->config['amun_service_path'] . '/' . $record->name . '/config.xml';

			if(is_file($config))
			{
				$xml = simplexml_load_file($config);

				if($xml === false)
				{
					throw new PSX_Data_Exception('Could not load config.xml');
				}
				else
				{
					// read meta data from config
					$this->parseConfigMeta($xml, $record);
				}
			}
			else
			{
				throw new PSX_Data_Exception('Found no config.xml');
			}


			$date = new DateTime('NOW', $this->registry['core.default_timezone']);

			$record->date = $date->format(PSX_Time::SQL);


			$this->table->insert($record->getData());


			$this->serviceId = $this->sql->getLastInsertId();


			// copy files
			$this->copyFiles($record->name);


			// execute all db actions in the config
			$this->parseConfigDb($xml);


			$record->id = $this->serviceId;

			$this->notify(Amun_Data_RecordAbstract::INSERT, $record);


			return $record;
		}
		else
		{
			throw new PSX_Data_Exception('Missing field in record');
		}
	}

	public function update(PSX_Data_RecordInterface $record)
	{
		throw new PSX_Data_Exception('Update a service record is not possible');
	}

	public function delete(PSX_Data_RecordInterface $record)
	{
		if($record->hasFields('id'))
		{
			$con  = new PSX_Sql_Condition(array('id', '=', $record->id));
			$name = $this->sql->select($this->table->getName(), array('name'), $con, PSX_Sql::SELECT_FIELD);

			if(!empty($name))
			{
				$config = $this->config['amun_service_path'] . '/' . $name . '/config.xml';

				if(is_file($config))
				{
					$xml = simplexml_load_file($config);

					if($xml !== false)
					{
						$this->parseXml($xml);
					}
				}

				$this->deleteFiles($module);
			}


			// delete options
			$con = new PSX_Sql_Condition(array('serviceId', '=', $record->id));

			$this->sql->delete($this->registry['table.content_service_option'], $con);


			$this->table->delete($con);


			$this->notify(Amun_Data_RecordAbstract::DELETE, $record);


			return $record;
		}
		else
		{
			throw new PSX_Data_Exception('missing field in record');
		}
	}

	private function copyFiles($name)
	{
		// copy API
		$this->copyFromTo(

			$this->config['amun_service_path'] . '/' . $name . '/api',
			PSX_PATH_MODULE . '/api/service/' . $name,
			array('index.php')

		);

		// copy API index
		$this->copyFromTo(

			$this->config['amun_service_path'] . '/' . $name . '/api/index.php',
			PSX_PATH_MODULE . '/api/service/' . $name . '.php'

		);

		// copy application
		$this->copyFromTo(

			$this->config['amun_service_path'] . '/' . $name . '/application',
			PSX_PATH_MODULE . '/application/' . $name

		);

		// copy gadget
		$this->copyFromTo(

			$this->config['amun_service_path'] . '/' . $name . '/gadget',
			PSX_PATH_MODULE . '/gadget/' . $name

		);

		// copy library
		$this->copyFromTo(

			$this->config['amun_service_path'] . '/' . $name . '/library',
			PSX_PATH_LIBRARY . '/Amun/Service/' . ucfirst($name),
			array('Index.php')

		);

		// copy library index
		$this->copyFromTo(

			$this->config['amun_service_path'] . '/' . $name . '/library/Index.php',
			PSX_PATH_LIBRARY . '/Amun/Service/' . ucfirst($name) . '.php'

		);

		// copy template
		$this->copyFromTo(

			$this->config['amun_service_path'] . '/' . $name . '/template',
			PSX_PATH_TEMPLATE . '/' . $this->config['psx_template_dir'] . '/application/' . $name

		);
	}

	private function copyFromTo($from, $to, array $exclude = array())
	{
		if(is_file($from) && !is_file($to))
		{
			if(!copy($from, $to))
			{
				throw new PSX_Data_Exception('Couldnt copy file to ' . $to);
			}
		}
		elseif(is_dir($from))
		{
			if(!is_dir($to))
			{
				if(mkdir($to))
				{
					if($this->copyRecDir($from, $to, $exclude) === false)
					{
						throw new PSX_Data_Exception('Could not copy files');
					}
				}
				else
				{
					throw new PSX_Data_Exception('Could not create dir ' . $to);
				}
			}
			else
			{
				if($this->copyRecDir($from, $to, $exclude) === false)
				{
					throw new PSX_Data_Exception('Could not copy files');
				}
			}
		}
	}

	private function deleteFiles($module)
	{
		// api
		$api = PSX_PATH_MODULE . '/api/service/' . $module;

		if(is_dir($api))
		{
			$this->deleteRecDir($api);

			rmdir($api);
		}


		// library
		$library = PSX_PATH_LIBRARY . '/amun/service/' . $module;

		if(is_dir($library))
		{
			$this->deleteRecDir($library);

			rmdir($library);
		}

		$index = PSX_PATH_LIBRARY . '/amun/service/' . $module . '.php';

		if(is_file($index))
		{
			unlink($index);
		}
	}

	private function parseConfigMeta(SimpleXMLElement $xml, Amun_Content_Service $record)
	{
		if(!isset($xml->status))
		{
			throw new PSX_Data_Exception('The service has no status');
		}
		else
		{
			$record->setStatus(strval($xml->status));
		}

		if(!isset($xml->name))
		{
			throw new PSX_Data_Exception('The service has no name');
		}
		else
		{
			$record->setName(strval($xml->name));
		}

		if(!isset($xml->type))
		{
			throw new PSX_Data_Exception('The service has no type');
		}
		else
		{
			$record->setType(strval($xml->type));
		}

		foreach(array('link', 'license', 'version') as $tag)
		{
			if(isset($xml->$tag))
			{
				$record->$tag = strval($xml->$tag);
			}
			else
			{
				$record->$tag = '';
			}
		}

		if(isset($xml->required) && isset($xml->required->service))
		{
			foreach($xml->required->service as $service)
			{
				$con = new PSX_Sql_Condition(array('name', '=', $service));

				if($this->sql->count($this->registry['table.content_service'], $con) == 0)
				{
					throw new PSX_Data_Exception('Requried service ' . $service . ' is not installed');
				}
			}
		}
	}

	private function parseConfigDb(SimpleXMLElement $xml)
	{
		if(isset($xml->database))
		{
			if(isset($xml->database->insert) && isset($xml->database->insert->table))
			{
				foreach($xml->database->insert->table as $table)
				{
					$this->parseInsert($table);
				}
			}


			// reload registry container before we create new tables etc
			// in case the content has changed
			$this->registry->load();


			if(isset($xml->database->create) && isset($xml->database->create->table))
			{
				foreach($xml->database->create->table as $table)
				{
					$this->parseCreate($table);
				}
			}

			if(isset($xml->database->alter))
			{
				if(isset($xml->database->alter->add) && isset($xml->database->alter->add->table))
				{
					foreach($xml->database->alter->add->table as $table)
					{
						$this->parseAlterAdd($table);
					}
				}

				if(isset($xml->database->alter->change) && isset($xml->database->alter->change->table))
				{
					foreach($xml->database->alter->change->table as $table)
					{
						$this->parseAlterChange($table);
					}
				}

				if(isset($xml->database->alter->modify) && isset($xml->database->alter->modify->table))
				{
					foreach($xml->database->alter->modify->table as $table)
					{
						$this->parseAlterModify($table);
					}
				}

				if(isset($xml->database->alter->drop) && isset($xml->database->alter->drop->table))
				{
					foreach($xml->database->alter->drop->table as $table)
					{
						$this->parseAlterDrop($table);
					}
				}
			}

			if(isset($xml->database->drop) && isset($xml->database->drop->table))
			{
				foreach($xml->database->drop->table as $table)
				{
					$this->parseDrop($table);
				}
			}

			if(isset($xml->database->truncate) && isset($xml->database->truncate->table))
			{
				foreach($xml->database->truncate->table as $table)
				{
					$this->parseTruncate($table);
				}
			}

			if(isset($xml->database->update) && isset($xml->database->update->table))
			{
				foreach($xml->database->update->table as $table)
				{
					$this->parseUpdate($table);
				}
			}

			if(isset($xml->database->delete) && isset($xml->database->delete->table))
			{
				foreach($xml->database->delete->table as $table)
				{
					$this->parseDelete($table);
				}
			}
		}
	}

	private function parseInsert(SimpleXMLElement $table)
	{
		$tblName = 'table.' . (isset($table['name']) ? strval($table['name']) : '');

		if(isset($this->registry[$tblName]))
		{
			$tblName = $this->registry[$tblName];
		}
		else
		{
			$tblName = false;
		}

		if($tblName !== false && isset($table->col))
		{
			$cols = array();

			foreach($table->col as $col)
			{
				$cols[] = strval($col);
			}

			if(isset($table->record) && isset($table->record->row))
			{
				$i = 0;

				foreach($table->record as $k => $record)
				{
					$rows = array();

					foreach($record->row as $row)
					{
						$row = strval($row);

						if(isset($this->ids[$row]))
						{
							$row = $this->ids[$row];
						}
						else if($row == '__NOW__')
						{
							$date = new DateTime('NOW', $this->registry['core.default_timezone']);

							$row = $date->format(PSX_Time::SQL);
						}
						else if($row == '__SERVICE_ID__')
						{
							$row = $this->serviceId;
						}

						$rows[] = $row;
					}

					if(count($cols) == count($rows))
					{
						$values = array_combine($cols, $rows);

						if($values !== false)
						{
							$this->sql->insert($tblName, $values);

							$this->ids['__' . strtoupper($table['name']) . '_' . $i . '__'] = $this->sql->getLastInsertId();

							$i++;
						}
					}
				}
			}
		}
	}

	private function parseCreate(SimpleXMLElement $table)
	{
		$tblName = 'table.' . (isset($table['name']) ? strval($table['name']) : '');

		if(isset($this->registry[$tblName]))
		{
			$tblName = $this->registry[$tblName];
		}
		else
		{
			$tblName = false;
		}

		if($tblName !== false && isset($table->col))
		{
			$cols = array();

			foreach($table->col as $col)
			{
				$cols[] = strval($col);
			}

			$sql  = 'CREATE TABLE IF NOT EXISTS `' . $tblName . '` (';
			$sql .= implode(', ', $cols);
			$sql .= ') ENGINE=MyISAM DEFAULT CHARSET=utf8';

			$this->sql->query($sql);
		}
	}

	private function parseAlterAdd(SimpleXMLElement $table)
	{
		$tblName = 'table.' . (isset($table['name']) ? strval($table['name']) : '');

		if($tblName !== false && isset($this->registry[$tblName]))
		{
			$tblName = $this->registry[$tblName];
		}
		{
			$tblName = false;
		}

		if($tblName !== false && isset($table->col))
		{
			$col = strval($table->col);
			$sql = 'ALTER TABLE `' . $tblName . '` ADD ' . $col;

			$this->sql->query($sql);
		}
	}

	private function parseAlterChange(SimpleXMLElement $table)
	{
		$tblName = 'table.' . (isset($table['name']) ? strval($table['name']) : '');

		if($tblName !== false && isset($this->registry[$tblName]))
		{
			$tblName = $this->registry[$tblName];
		}
		else
		{
			$tblName = false;
		}

		if($tblName !== false && isset($table->col))
		{
			$col = strval($table->col);
			$sql = 'ALTER TABLE `' . $tblName . '` CHANGE ' . $col;

			$this->sql->query($sql);
		}
	}

	private function parseAlterModify(SimpleXMLElement $table)
	{
		$tblName = 'table.' . (isset($table['name']) ? strval($table['name']) : '');

		if($tblName !== false && isset($this->registry[$tblName]))
		{
			$tblName = $this->registry[$tblName];
		}
		else
		{
			$tblName = false;
		}

		if($tblName !== false && isset($table->col))
		{
			$col = strval($table->col);
			$sql = 'ALTER TABLE `' . $tblName . '` MODIFY ' . $col;

			$this->sql->query($sql);
		}
	}

	private function parseAlterDrop(SimpleXMLElement $table)
	{
		$tblName = 'table.' . (isset($table['name']) ? strval($table['name']) : '');

		if($tblName !== false && isset($this->registry[$tblName]))
		{
			$tblName = $this->registry[$tblName];
		}
		else
		{
			$tblName = false;
		}

		if($table !== false && isset($table->col))
		{
			$col = strval($table->col);
			$sql = 'ALTER TABLE `' . $tblName . '` DROP ' . $col;

			$this->sql->query($sql);
		}
	}

	private function parseDrop(SimpleXMLElement $table)
	{
		$tblName = 'table.' . (isset($table['name']) ? strval($table['name']) : '');

		if($tblName !== false && isset($this->registry[$tblName]))
		{
			$tblName = $this->registry[$tblName];
		}
		else
		{
			$tblName = false;
		}

		if($tblName !== false)
		{
			$sql = 'DROP TABLE `' . $tblName . '`';

			$this->sql->query($sql);
		}
	}

	private function parseTruncate(SimpleXMLElement $table)
	{
		$tblName = 'table.' . (isset($table['name']) ? strval($table['name']) : '');

		if($tblName !== false && isset($this->registry[$tblName]))
		{
			$tblName = $this->registry[$tblName];
		}
		else
		{
			$tblName = false;
		}

		if($tblName !== false)
		{
			$sql = 'TRUNCATE TABLE `' . $tblName . '`';

			$this->sql->query($sql);
		}
	}

	private function parseUpdate(SimpleXMLElement $table)
	{
		$tblName = 'table.' . (isset($table['name']) ? strval($table['name']) : '');

		if($tblName !== false && isset($this->registry[$tblName]))
		{
			$tblName = $this->registry[$tblName];
		}
		else
		{
			$tblName = false;
		}

		if($tblName !== false && isset($table['col']) && isset($table['value']) && isset($table->col))
		{
			$cols = array();

			foreach($table->col as $col)
			{
				$cols[] = strval($col);
			}

			if(isset($table->record) && isset($table->record->row))
			{
				$rows   = (array) $table->record->row;
				$values = array_combine($cols, $rows);

				if($values !== false)
				{
					$this->sql->update($tblName, $table['col'], $table['value'], $values);
				}
			}
		}
	}

	private function parseDelete(SimpleXMLElement $table)
	{
		$tblName = 'table.' . (isset($table['name']) ? strval($table['name']) : '');

		if($tblName !== false && isset($this->registry[$tblName]))
		{
			$tblName = $this->registry[$tblName];
		}
		else
		{
			$tblName = false;
		}

		if($tblName !== false && isset($table['col']) && isset($table['value']))
		{
			$this->sql->delete($tblName, $table['col'], $table['value']);
		}
	}

	private function copyRecDir($source, $dest, array $exclude = array())
	{
		$dirs = scandir($source);

		foreach($dirs as $item)
		{
			if($item != '.' && $item != '..' && $item[0] != '_' && !in_array($item, $exclude))
			{
				$s = $source . '/' . $item;
				$d = $dest . '/' . $item;

				if(is_dir($s) && $item[0] != '.')
				{
					if(!is_dir($d))
					{
						if(mkdir($d))
						{
							if($this->copyRecDir($s, $d) === false)
							{
								return false;
							}
						}
						else
						{
							return false;
						}
					}
					else
					{
						if($this->copyRecDir($s, $d) === false)
						{
							return false;
						}
					}
				}

				if(is_file($s))
				{
					if(!is_file($d))
					{
						if(!copy($s, $d))
						{
							return false;
						}
					}
				}
			}
		}

		return true;
	}

	private function deleteRecDir($source)
	{
		$dirs = scandir($source);

		foreach($dirs as $item)
		{
			if($item != '.' && $item != '..')
			{
				$s = $source . '/' . $item;

				if(is_dir($s))
				{
					$this->deleteRecDir($s);

					rmdir($s);
				}

				if(is_file($s))
				{
					unlink($s);
				}
			}
		}
	}
}
