<?php
/*
 *  $Id: UrlTitle.php 222 2011-04-24 10:34:21Z k42b3.x $
 *
 * amun
 * A social content managment system based on the psx framework. For
 * the current version and informations visit <http://amun.phpsx.org>
 *
 * Copyright (c) 2010 Christoph Kappestein <k42b3.x@gmail.com>
 *
 * This file is part of amun. amun is free software: you can
 * redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or any later version.
 *
 * amun is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with amun. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * Amun_Filter_UrlTitle
 *
 * This filter converts an title to an urlTitle wich can be used in an url. We
 * use the punycode algorithm from rfc3492 to convert a title wich maybe has
 * non-ASCII signs (> 128) to an string wich includes only valid url chars. For
 * more nfos read the rfc3492.
 *
 * @author     Christoph Kappestein <k42b3.x@gmail.com>
 * @license    http://www.gnu.org/licenses/gpl.html GPLv3
 * @link       http://amun.phpsx.org
 * @category   Amun
 * @package    Amun_Filter
 * @version    $Revision: 222 $
 */
class Amun_Filter_UrlTitle extends PSX_FilterAbstract
{
	private $punny_code_prefix     = 'xn--';
	private $basic_code_points     = 0;
	private $non_basic_code_points = 0;
	private $basic_signs           = '';

	private $base         = 36;
	private $tmin         = 1;
	private $tmax         = 26;
	private $skew         = 38;
	private $damp         = 700;
	private $initial_bias = 72;
	private $initial_n    = 128;

	public function apply($value)
	{
		return $this->encode(strtolower($value));
	}

	/**
	 * encode
	 *
	 * This function is a simple port from the pseudocode of the RFC to PHP.
	 *
	 * @param string $string
	 */
	private function encode($string)
	{
		$chars = $this->buildCharArray($string);

		$out   = '';
		$len   = count($chars);
		$n     = $this->initial_n;
		$delta = 0;
		$bias  = $this->initial_bias;
		$h     = $b = $this->basic_code_points;
		$out  .= $this->punny_code_prefix . $this->basic_signs;

		if($this->basic_code_points == $len)
		{
			$str = '';

			foreach($chars as $v)
			{
				$str.= chr($v);
			}

			return $str;
		}

		if($b > 0)
		{
			$out.= '-';
		}

		while($h < $len)
		{
			$min = false;

			for($i = 0; $i < $len; $i++)
			{
				if(($min === false || $chars[$i] < $min) && $chars[$i] >= $n)
				{
					$min = $chars[$i];
				}
			}

			$m     = $min;
			$delta = $delta + ($m - $n) * ($h + 1);
			$n     = $m;

			foreach($chars as $c)
			{
				if($c < $n)
				{
					$delta++;
				}

				if($c == $n)
				{
					$q = $delta;
					$k = 0;

					while(true)
					{
						$k+= $this->base;

						$t = ($k <= $bias) ? $this->tmin : (($k >= $bias + $this->tmax) ? $this->tmax : $k - $bias);

						if($q < $t)
						{
							break;
						}

						$out.= $this->encodeDigit(intval($t + (($q - $t) % ($this->base - $t))));

						$q = ($q - $t) / ($this->base - $t);
					}

					$out.= $this->encodeDigit($q);

					$bias = $this->adapt($delta, $h + 1, $h == $b);

					$delta = 0;

					$h++;
				}
			}

			$delta++;
			$n++;
		}

		return $out;
	}

	private function adapt($delta, $numpoints, $firsttime)
	{
		if($firsttime)
		{
			$delta = $delta / $this->damp;
		}
		else
		{
			$delta = $delta / 2;
		}

		$delta = $delta + ($delta / $numpoints);
		$k     = 0;

		while($delta > (($this->base - $this->tmin) * $this->tmax) / 2)
		{
			$delta = $delta / ($this->base - $this->tmin);
			$k     = $k + $this->base;
		}

		return $k + ((($this->base - $this->tmin + 1) * $delta) / ($delta + $this->skew));
	}

	/**
	 * buildCharArray
	 *
	 * The function converts an string into an ascii array and counts all
	 * basic and non basic codes. If the string contains an non basic code
	 * lower then 128 then the value is skipped.
	 *
	 * @param string $string
	 */
	private function buildCharArray($string)
	{
		$char = array();
		$i    = 0;
		$len  = strlen($string);

		for($i = 0; $i < $len; $i++)
		{
			$ascii = ord($string[$i]);

			   # 0 - 9
			if(($ascii >= 0x30 && $ascii <= 0x39) ||
			   # A - Z
			   ($ascii >= 0x41 && $ascii <= 0x5A) ||
			   # a - z
			   ($ascii >= 0x61 && $ascii <= 0x7A) ||
			   # -
			   ($ascii == 0x2D))
			{
				$this->basic_code_points++;

				$this->basic_signs.= $string[$i];

				$char[] = $ascii;
			}
			elseif($ascii >= $this->initial_n)
			{
				$this->non_basic_code_points++;

				$char[] = $ascii;
			}
		}

		return $char;
	}

	private function encodeDigit($d)
	{
		return chr($d + 22 + 75 * ($d < 26));
	}
}