<?php
/*
 *  $Id: Item.php 403 2011-11-15 22:07:13Z k42b3.x@googlemail.com $
 *
 * amun
 * A social content managment system based on the psx framework. For
 * the current version and informations visit <http://amun.phpsx.org>
 *
 * Copyright (c) 2010 Christoph Kappestein <k42b3.x@gmail.com>
 *
 * This file is part of amun. amun is free software: you can
 * redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or any later version.
 *
 * amun is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with amun. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * Amun_Gadget_Item
 *
 * @author     Christoph Kappestein <k42b3.x@gmail.com>
 * @license    http://www.gnu.org/licenses/gpl.html GPLv3
 * @link       http://amun.phpsx.org
 * @category   Amun
 * @package    Amun_Gadget
 * @version    $Revision: 403 $
 */
class Amun_Gadget_Item
{
	public $id;
	public $title;
	public $path;
	public $cache;
	public $expire;

	protected $config;
	protected $application;

	protected $sql;
	protected $registry;
	protected $validate;
	protected $get;
	protected $post;

	protected $session;
	protected $user;
	protected $page;
	/*
	protected $nav;
	protected $path;
	protected $gadget;
	*/

	protected $htmlJs;
	protected $htmlCss;
	protected $htmlContent;

	private $body;

	public function __construct(Amun_Registry $registry, Amun_Base_Application $application)
	{
		$this->config      = $registry->getConfig();
		$this->application = $application;

		$this->sql         = $application->getSql();
		$this->registry    = $application->getRegistry();
		$this->validate    = $application->getValidate();
		$this->get         = $application->getGet();
		$this->post        = $application->getPost();

		$this->session     = $application->getSession();
		$this->user        = $application->getUser();
		$this->page        = $application->getPage();
		/*
		$this->nav         = $application->getNav();
		$this->path        = $application->getPath();
		$this->gadget      = $application->getGadget();
		*/

		$this->htmlJs      = $application->getHtmlJs();
		$this->htmlCss     = $application->getHtmlCss();
		$this->htmlContent = $application->getHtmlContent();
	}

	public function getTitle()
	{
		return $this->title;
	}

	public function getBody()
	{
		return $this->body;
	}

	public function setBody($body)
	{
		$this->body = $body;
	}

	public function parseContent()
	{
		$handler = new PSX_Cache_Handler_File();
		$key     = 'c_g_' . md5($this->id) . '.htm';
		$cache   = new PSX_Cache($key, $handler, $this->config);

		if($this->cache == 0 || ($content = $cache->load($key, $this->expire)) === false)
		{
			$content = $this->executeContent($this->path);

			// if caching is enabled write the cache
			if($this->cache == 1)
			{
				$cache->write($content);
			}
		}

		$this->setBody($content);
	}

	private function executeContent($path)
	{
		ob_start();

		include(PSX_PATH_MODULE . '/gadget/' . $path);

		$content = ob_get_contents();

		ob_end_clean();


		return $content;
	}

	private function getCache($token, $expire)
	{
		$file = PSX_PATH_CACHE . '/c_g_' . $token . '.htm';

		if(is_file($file))
		{
			if((filemtime($file) + $expire) > time())
			{
				$content = file_get_contents($file);

				return $content;
			}
		}

		return false;
	}
}