<?php
/*
 *  $Id: Registry.php 450 2011-12-04 22:35:35Z k42b3.x@googlemail.com $
 *
 * amun
 * A social content managment system based on the psx framework. For
 * the current version and informations visit <http://amun.phpsx.org>
 *
 * Copyright (c) 2010 Christoph Kappestein <k42b3.x@gmail.com>
 *
 * This file is part of amun. amun is free software: you can
 * redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or any later version.
 *
 * amun is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with amun. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * Amun_Registry
 *
 * @author     Christoph Kappestein <k42b3.x@gmail.com>
 * @license    http://www.gnu.org/licenses/gpl.html GPLv3
 * @link       http://amun.phpsx.org
 * @category   Amun
 * @package    Amun_Registry
 * @version    $Revision: 450 $
 */
class Amun_Registry extends PSX_Registry
{
	protected $config;
	protected $sql;

	public function __construct(PSX_Config $config, PSX_Sql $sql)
	{
		parent::__construct();

		$this->config = $config;
		$this->sql    = $sql;

		$this->load();
	}

	public function getConfig()
	{
		return $this->config;
	}

	public function getSql()
	{
		return $this->sql;
	}

	public function hasService($name)
	{
		$con   = new PSX_Sql_Condition(array('name', '=', $name));
		$count = $this->sql->count($this->get('table.content_service'), $con);

		return $count > 0;
	}

	public function load()
	{
		$result = $this->sql->getAll('SELECT `name`, `type`, `class`, `value` FROM ' . $this->config['amun_table_registry']);

		foreach($result as $row)
		{
			switch($row['type'])
			{
				case 'STRING':

					$value = (string) $row['value'];

					break;

				case 'INTEGER':

					$value = (integer) $row['value'];

					break;

				case 'FLOAT':

					$value = (float) $row['value'];

					break;

				case 'BOOLEAN':

					$value = (boolean) $row['value'];

					break;
			}

			if(!empty($row['class']) && class_exists($row['class']))
			{
				$class = $row['class'];
				$value = new $class($value);
			}

			$this->offsetSet($row['name'], $value);
		}
	}

	public function getClassNameFromTable($table)
	{
		$it = $this->getIterator();

		while($it->valid())
		{
			if(substr($it->key(), 0, 6) == 'table.' && $it->current() == $table)
			{
				return self::getClassName(substr($it->key(), 6));
			}

			$it->next();
		}

		return false;
	}

	public static function initInstance(PSX_Config $config, PSX_Sql $sql)
	{
		return self::$_instance = new self($config, $sql);
	}

	public static function getClassName($table)
	{
		return implode('_', array_map('ucfirst', explode('_', $table)));
	}
}


