<?php
/*
 *  $Id: Connect.php 416 2011-11-19 22:01:44Z k42b3.x@googlemail.com $
 *
 * amun
 * A social content managment system based on the psx framework. For
 * the current version and informations visit <http://amun.phpsx.org>
 *
 * Copyright (c) 2010 Christoph Kappestein <k42b3.x@gmail.com>
 *
 * This file is part of amun. amun is free software: you can
 * redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or any later version.
 *
 * amun is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with amun. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * Amun_System_Connect
 *
 * @author     Christoph Kappestein <k42b3.x@gmail.com>
 * @license    http://www.gnu.org/licenses/gpl.html GPLv3
 * @link       http://amun.phpsx.org
 * @category   Amun
 * @package    Amun_System_Connect
 * @version    $Revision: 416 $
 */
class Amun_System_Connect extends Amun_Data_RecordAbstract
{
	const TEMPORARY = 0x1;
	const APPROVED  = 0x2;
	const ACCESS    = 0x3;

	protected $_user;
	protected $_date;

	public function setId($id)
	{
		$id = $this->_validate->apply($id, 'integer', array(new Amun_Filter_Id($this->_table)), 'id', 'Id');

		if(!$this->_validate->hasError())
		{
			$this->id = $id;
		}
		else
		{
			throw new PSX_Data_Exception($this->_validate->getLastError());
		}
	}

	public function setUserId($userId)
	{
		$userId = $this->_validate->apply($userId, 'integer', array(new Amun_Filter_Id(Amun_Sql_Table_Registry::get('User_Account'))), 'userId', 'User Id');

		if(!$this->_validate->hasError())
		{
			$this->userId = $userId;
		}
		else
		{
			throw new PSX_Data_Exception($this->_validate->getLastError());
		}
	}

	public function setStatus($status)
	{
		$status = $this->_validate->apply($status, 'integer', array(new Amun_System_Connect_Filter_Status()), 'status', 'Status');

		if(!$this->_validate->hasError())
		{
			$this->status = $status;
		}
		else
		{
			throw new PSX_Data_Exception($this->_validate->getLastError());
		}
	}

	public function setClaimedId($claimedId)
	{
		$claimedId = $this->_validate->apply($claimedId, 'string', array(new PSX_Filter_Length(3, 256), new PSX_Filter_Url()), 'claimedId', 'Claimed Id');

		if(!$this->_validate->hasError())
		{
			$this->claimedId = $claimedId;
		}
		else
		{
			throw new PSX_Data_Exception($this->_validate->getLastError());
		}
	}

	public function setIdentity($identity)
	{
		$identity = $this->_validate->apply($identity, 'string', array(new PSX_Filter_Length(3, 256), new PSX_Filter_Url()), 'identity', 'Identity');

		if(!$this->_validate->hasError())
		{
			$this->identity = $identity;
		}
		else
		{
			throw new PSX_Data_Exception($this->_validate->getLastError());
		}
	}

	public function setReturnTo($returnTo)
	{
		$returnTo = $this->_validate->apply($returnTo, 'string', array(new PSX_Filter_Length(3, 256), new PSX_Filter_Url()), 'returnTo', 'Return To');

		if(!$this->_validate->hasError())
		{
			$this->returnTo = $returnTo;
		}
		else
		{
			throw new PSX_Data_Exception($this->_validate->getLastError());
		}
	}

	public function setResponseNonce($responseNonce)
	{
		$this->responseNonce = $responseNonce;
	}

	public function setAssocHandle($assocHandle)
	{
		$this->assocHandle = $assocHandle;
	}

	public function setSig($sig)
	{
		$this->sig = $sig;
	}

	public function setSecret($secret)
	{
		$this->secret = $secret;
	}

	public function setExpire($expire)
	{
		$this->expire = $expire;
	}

	public function getId()
	{
		return $this->_config->getUrn('system', 'connect', $this->id);
	}

	public function getUser()
	{
		if($this->_user === null)
		{
			$this->_user = Amun_Sql_Table_Registry::get('User_Account')->getRecord($this->userId);
		}

		return $this->_user;
	}

	public function getDate()
	{
		if($this->_date === null)
		{
			$this->_date = new DateTime($this->date, $this->_registry['core.default_timezone']);
		}

		return $this->_date;
	}

	public static function getStatus($status = false)
	{
		$s = array(

			self::TEMPORARY => 'Temporary',
			self::APPROVED  => 'Approved',
			self::ACCESS    => 'Access',

		);

		if($status !== false)
		{
			$status = intval($status);

			if(array_key_exists($status, $s))
			{
				return $s[$status];
			}
			else
			{
				return false;
			}
		}
		else
		{
			return $s;
		}
	}
}


