<?php
/*
 *  $Id: Handler.php 432 2011-12-03 18:11:46Z k42b3.x@googlemail.com $
 *
 * amun
 * A social content managment system based on the psx framework. For
 * the current version and informations visit <http://amun.phpsx.org>
 *
 * Copyright (c) 2010 Christoph Kappestein <k42b3.x@gmail.com>
 *
 * This file is part of amun. amun is free software: you can
 * redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or any later version.
 *
 * amun is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with amun. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * Amun_User_Friend_Handler
 *
 * @author     Christoph Kappestein <k42b3.x@gmail.com>
 * @license    http://www.gnu.org/licenses/gpl.html GPLv3
 * @link       http://amun.phpsx.org
 * @category   Amun
 * @package    Amun_User_Friend
 * @version    $Revision: 432 $
 */
class Amun_User_Friend_Handler extends Amun_Data_HandlerAbstract
{
	public function create(PSX_Data_RecordInterface $record)
	{
		if($record->hasFields('friendId'))
		{
			$record->userId = $this->user->id;


			// check id
			if($this->user->id == $record->friendId)
			{
				throw new PSX_Data_Exception('You can not establish a relation to yourself');
			}


			// determine status
			$record->status = $this->getStatus($record);


			// check whether friend is an remote account if yes send an
			// friend request to the remote host
			$this->handleRemoteSubscription($record);


			$date = new DateTime('NOW', $this->registry['core.default_timezone']);

			$record->date = $date->format(PSX_Time::SQL);


			$this->table->insert($record->getData());

			$friendId = $this->sql->getLastInsertId();


			// build relation
			if($record->status == Amun_User_Friend::NORMAL)
			{
				$con = new PSX_Sql_Condition();
				$con->add('userId', '=', $record->friendId);
				$con->add('friendId', '=', $this->user->id);

				$this->table->update(array(

					'status' => Amun_User_Friend::NORMAL,
					'date'   => $date->format(PSX_Time::SQL),

				), $con);
			}


			$record->id = $friendId;

			$this->notify(Amun_Data_RecordAbstract::INSERT, $record);


			return $record;
		}
		else
		{
			throw new PSX_Data_Exception('Missing field in record');
		}
	}

	public function update(PSX_Data_RecordInterface $record)
	{
		throw new PSX_Data_Exception('Friend can not updated');
	}

	public function delete(PSX_Data_RecordInterface $record)
	{
		if($record->hasFields('id'))
		{
			$con = new PSX_Sql_Condition(array('id', '=', $record->id));

			$this->table->delete($con);


			$this->notify(Amun_Data_RecordAbstract::DELETE, $record);


			return $record;
		}
		else
		{
			throw new PSX_Data_Exception('Missing field in record');
		}
	}

	public function relation(Amun_User_Friend_Relation $record)
	{
		if($record->hasFields('mode', 'host', 'name'))
		{
			switch($record->mode)
			{
				case 'request':

					return $this->request($record);

					break;

				case 'accept':

					return $this->accept($record);

					break;

				case 'deny':

					return $this->deny($record);

					break;
			}
		}
		else
		{
			throw new PSX_Data_Exception('Missing field in record');
		}
	}

	private function request(Amun_User_Friend_Relation $record)
	{
		$sql = <<<SQL
SELECT

	`host`.`id`       AS `hostId`,
	`host`.`name`     AS `hostName`,
	`host`.`template` AS `hostTemplate`

	FROM {$this->registry['table.system_host']} `host`

		WHERE `host`.`name` = ?
SQL;

		$row = $this->sql->getRow($sql, array($record->host));

		if(!empty($row))
		{
			// request profile url
			$http       = new PSX_Http(new PSX_Http_Handler_Curl());
			$webfinger  = new PSX_Webfinger($http);

			$email      = $record->name . '@' . $row['hostName'];
			$profileUrl = $webfinger->getAcctProfile($email, $row['hostTemplate']);
			$identity   = PSX_OpenId::normalizeIdentifier($profileUrl);


			// create remote user if not exists
			$con      = new PSX_Sql_Condition(array('identity', '=', sha1(Amun_Security::getSalt() . $identity)));
			$friendId = $this->sql->select($this->registry['table.user_account'], array('id'), $con, PSX_Sql::SELECT_FIELD);

			if(empty($friendId))
			{
				$handler = new Amun_User_Account_Handler($this->user);

				$account = Amun_Sql_Table_Registry::get('User_Account')->getRecord();
				$account->setGroupId($this->registry['core.default_user_group']);
				$account->setHostId($row['hostId']);
				$account->setStatus(Amun_User_Account::REMOTE);
				$account->setIdentity($identity);
				$account->setName($record->name);
				$account->setPw(Amun_Security::generatePw());

				$account  = $handler->create($account);
				$friendId = $account->id;
			}


			// create relation
			$friend = Amun_Sql_Table_Registry::get('User_Friend')->getRecord();
			$friend->friendId = $friendId;

			return $this->create($friend);
		}
		else
		{
			throw new PSX_Data_Exception('Invalid host');
		}
	}

	private function accept(Amun_User_Friend_Relation $record)
	{
		$sql = <<<SQL
SELECT

	`account`.`id`    AS `accountId`,
	`host`.`id`       AS `hostId`,
	`host`.`name`     AS `hostName`,
	`host`.`template` AS `hostTemplate`

	FROM {$this->registry['table.user_account']} `account`

		INNER JOIN {$this->registry['table.system_host']} `host`

		ON `account`.`hostId` = `host`.`id`

			WHERE `account`.`name` = ?

			AND `host`.`name` = ?

				AND `account`.`status` = ?
SQL;

		$row = $this->sql->getRow($sql, array($record->name, $record->host, Amun_User_Account::REMOTE));

		if(!empty($row))
		{
			// create relation
			$date = new DateTime('NOW', $this->registry['core.default_timezone']);

			$this->table->insert(array(

				'status'   => Amun_User_Friend::NORMAL,
				'userId'   => $row['accountId'],
				'friendId' => $this->user->id,
				'date'     => $date->format(PSX_Time::SQL),

			));


			// update status
			$con = new PSX_Sql_Condition();
			$con->add('userId', '=', $this->user->id);
			$con->add('friendId', '=', $row['accountId']);

			$this->table->update(array(

				'status' => Amun_User_Friend::NORMAL,
				'date'   => $date->format(PSX_Time::SQL),

			), $con);


			return true;
		}
		else
		{
			throw new PSX_Data_Exception('Account does not exist');
		}
	}

	private function deny(Amun_User_Friend_Relation $record)
	{
		throw new PSX_Data_Exception('Not implemented yet');
	}

	private function getStatus(PSX_Data_RecordInterface $record)
	{
		$con = new PSX_Sql_Condition();
		$con->add('userId', '=', $record->friendId);
		$con->add('friendId', '=', $record->userId);

		$count = $this->table->count($con);

		return $count > 0 ? Amun_User_Friend::NORMAL : Amun_User_Friend::REQUEST;
	}

	private function handleRemoteSubscription(PSX_Data_RecordInterface $record)
	{
		$cred = null;

		switch($record->status)
		{
			// a remote user wants to request a user as friend. We must notify
			// the remote wensite about the friendship request
			case Amun_User_Friend::REQUEST:

				if($record->getUser()->status == Amun_User_Account::REMOTE)
				{
					$url  = new PSX_Url($record->getUser()->getHost()->url);
					$mode = Amun_Relation::REQUEST;
					$host = $this->config->getHost();
					$name = $record->getFriend()->name;
					$cred = $record->getUser()->getRemoteCredentials();
				}

				break;

			// a user accepted a friendship request where the initiator was an
			// remote user we must inform the remote website that the request
			// was accepted
			case Amun_User_Friend::NORMAL:

				if($record->getFriend()->status == Amun_User_Account::REMOTE)
				{
					$url  = new PSX_Url($record->getFriend()->getHost()->url);
					$mode = Amun_Relation::ACCEPT;
					$host = $this->config->getHost();
					$name = $record->getUser()->name;
					$cred = $record->getFriend()->getRemoteCredentials();
				}

				break;
		}

		if($cred instanceof PSX_Oauth_Provider_Data_Consumer)
		{
			$http  = new PSX_Http(new PSX_Http_Handler_Curl());
			$oauth = new PSX_Oauth($http);

			$relation = new Amun_Relation($http, $cred);
			$relation->request($url, $mode, $host, $name);
		}
	}
}


