<?php
/*
 *  $Id: Cookie.php 354 2011-10-29 15:25:33Z k42b3.x@googlemail.com $
 *
 * psx
 * A object oriented and modular based PHP framework for developing
 * dynamic web applications. For the current version and informations
 * visit <http://phpsx.org>
 *
 * Copyright (c) 2011 Christoph Kappestein <k42b3.x@gmail.com>
 *
 * This file is part of psx. psx is free software: you can
 * redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or any later version.
 *
 * psx is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with psx. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * PSX_Cookie
 *
 * A class to set get and delete cookies. See the PSX_InputAbstract class how
 * you can access values and use the PSX_Validate class. Here a short example
 * how to get cookie value.
 * <code>
 * $cookie = new PSX_Cookie(new PSX_Validate());
 *
 * // set the data type
 * $foo = $cookie->foo('integer');
 *
 * // use the validation class
 * $foo = $cookie->foo('integer', new PSX_Filter_Url());
 *
 * // get the raw cookie value
 * $foo = $cookie->foo;
 * </code>
 *
 * You can set a cookie with the method "set". The method accept the same
 * parameters like the setcookie function see the manual at
 * http://php.net/manual/function.setcookie.php for more informations
 * <code>
 * $cookie->set('bar', 'foo');
 *
 * // or
 * $cookie['bar'] = 'foo';
 *
 * // to set a cookie that expire after an hour
 * $cookie->set('bar', 'foo', 60 * 60);
 * </code>
 *
 * To delete a cookie you use the method "delete". The method overwrites the
 * cookie with an empty cookie that expires in the past so the browser should
 * delete it.
 * <code>
 * $cookie->delete('foo');
 *
 * // or
 * unset($cookie['foo']);
 * </code>
 *
 * @author     Christoph Kappestein <k42b3.x@gmail.com>
 * @license    http://www.gnu.org/licenses/gpl.html GPLv3
 * @link       http://phpsx.org
 * @category   PSX
 * @package    PSX_Cookie
 * @version    $Revision: 354 $
 */
class PSX_Cookie extends PSX_InputAbstract
{
	public function has($key)
	{
		return isset($_COOKIE[$key]);
	}

	public function get($key)
	{
		if($this->has($key))
		{
			return $_COOKIE[$key];
		}
		else
		{
			return false;
		}
	}

	public function set($name, $value = '', $expire = 0, $path = '', $domain = '', $secure = false, $httponly = false)
	{
		$_COOKIE[$name] = $value;

		if(!headers_sent())
		{
			setcookie($name, $value, $expire, $path, $domain, $secure, $httponly);
		}
	}

	public function delete($name)
	{
		if($this->has($name))
		{
			$_COOKIE[$name] = null;

			$this->set($name, '', time() - 3600);
		}
	}

	public function getContainer()
	{
		return $_COOKIE;
	}
}

