<?php
/*
 *  $Id: HandlerAbstract.php 376 2011-11-23 23:40:31Z k42b3.x@googlemail.com $
 *
 * psx
 * A object oriented and modular based PHP framework for developing
 * dynamic web applications. For the current version and informations
 * visit <http://phpsx.org>
 *
 * Copyright (c) 2011 Christoph Kappestein <k42b3.x@gmail.com>
 *
 * This file is part of psx. psx is free software: you can
 * redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or any later version.
 *
 * psx is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with psx. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * PSX_Data_HandlerAbstract
 *
 * @author     Christoph Kappestein <k42b3.x@gmail.com>
 * @license    http://www.gnu.org/licenses/gpl.html GPLv3
 * @link       http://phpsx.org
 * @category   PSX
 * @package    PSX_Data
 * @version    $Revision: 376 $
 */
abstract class PSX_Data_HandlerAbstract implements PSX_Data_HandlerInterface
{
	protected $table;

	public function __construct(PSX_Data_TableInterface $table)
	{
		$this->table = $table;
	}

	public function insert(PSX_Data_RecordInterface $record)
	{
		$fields = array_intersect_key($record->getFields(), $this->table->getColumns());

		if(!empty($fields))
		{
			$this->table->insert($fields);
		}
		else
		{
			throw new PSX_Data_Exception('No valid field');
		}
	}

	public function update(PSX_Data_RecordInterface $record)
	{
		$pk     = $this->table->getPrimaryKey();
		$fields = array_intersect_key($record->getFields(), $this->table->getColumns());

		if(!empty($fields) && isset($record->$pk))
		{
			$con = new PSX_Sql_Condition($pk, '=', $record->$pk);

			$this->table->update($fields, $con);
		}
		else
		{
			throw new PSX_Data_Exception('No valid field or primary key not set');
		}
	}

	public function delete(PSX_Data_RecordInterface $record)
	{
		$pk     = $this->table->getPrimaryKey();

		if(isset($record->$pk))
		{
			$con = new PSX_Sql_Condition($pk, '=', $record->$pk);

			$this->table->delete($con);
		}
		else
		{
			throw new PSX_Data_Exception('No primary key set');
		}
	}
}

