<?php
/*
 *  $Id: Curl.php 324 2011-07-13 21:39:22Z k42b3.x@googlemail.com $
 *
 * psx
 * A object oriented and modular based PHP framework for developing
 * dynamic web applications. For the current version and informations
 * visit <http://phpsx.org>
 *
 * Copyright (c) 2011 Christoph Kappestein <k42b3.x@gmail.com>
 *
 * This file is part of psx. psx is free software: you can
 * redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or any later version.
 *
 * psx is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with psx. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * PSX_Http_Handler_Curl
 *
 * @author     Christoph Kappestein <k42b3.x@gmail.com>
 * @license    http://www.gnu.org/licenses/gpl.html GPLv3
 * @link       http://phpsx.org
 * @category   PSX
 * @package    PSX_Http
 * @version    $Revision: 324 $
 */
class PSX_Http_Handler_Curl implements PSX_Http_HandlerInterface
{
	private $lastError;
	private $request;
	private $response;

	private $hasFollowLocation;

	public function __construct()
	{
		// CURLOPT_FOLLOWLOCATION throws an error if in safe mode or
		// open_basedir is set. Set this to false if CURLOPT_FOLLOWLOCATION is
		// not available on your system
		$this->hasFollowLocation = true;
	}

	public function setFollowLocation($followLocation)
	{
		$this->hasFollowLocation = (boolean) $followLocation;
	}

	public function request(PSX_Http_Request $request, $count = 0)
	{
		$handle = curl_init($request->getUrl()->__toString());

		curl_setopt($handle, CURLOPT_HEADER, true);
		curl_setopt($handle, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($handle, CURLOPT_CUSTOMREQUEST, $request->getMethod());
		curl_setopt($handle, CURLINFO_HEADER_OUT, true);


		$header = $request->getHeader();

		if(!empty($header))
		{
			$rawHeader = array();

			foreach($header as $k => $v)
			{
				$rawHeader[] = $k . ': ' . $v;
			}

			curl_setopt($handle, CURLOPT_HTTPHEADER, $rawHeader);
		}


		$body = $request->getBody();

		if(!empty($body))
		{
			curl_setopt($handle, CURLOPT_POSTFIELDS, $body);
		}


		if($request->getFollowLocation())
		{
			if($this->hasFollowLocation)
			{
				curl_setopt($handle, CURLOPT_FOLLOWLOCATION, true);
				curl_setopt($handle, CURLOPT_MAXREDIRS, $request->getMaxRedirects());
			}
		}


		if($request->isSSL())
		{
			$path = realpath(dirname(realpath(__FILE__)) . '/../ca-bundle.pem');

			curl_setopt($handle, CURLOPT_CAINFO, $path);
			curl_setopt($handle, CURLOPT_SSL_VERIFYPEER, true);
		}
		else
		{
			curl_setopt($handle, CURLOPT_SSL_VERIFYPEER, false);
			curl_setopt($handle, CURLOPT_SSL_VERIFYHOST, false);
		}


		$timeout = $request->getTimeout();

		if(!empty($timeout))
		{
			curl_setopt($handle, CURLOPT_CONNECTTIMEOUT, $timeout);
		}


		$callback = $request->getCallback();

		if(!empty($callback))
		{
			call_user_func_array($callback, array($handle, $request));
		}


		$response = curl_exec($handle);


		if(!curl_errno($handle))
		{
			$this->lastError = false;
			$this->request   = curl_getinfo($handle, CURLINFO_HEADER_OUT);
			$this->response  = $response;
		}
		else
		{
			$this->lastError = curl_error($handle);
			$this->request   = false;
			$this->response  = false;
		}

		curl_close($handle);


		if($request->getFollowLocation())
		{
			if(!$this->hasFollowLocation)
			{
				// is follow location doesnt work we must make the request again
				// if an location header is set
				$startPos = stripos($response, PSX_Http::$newLine . 'Location:');

				if($startPos !== false)
				{
					$startPos = $startPos + strlen(PSX_Http::$newLine . 'Location:');
					$location = substr($response, $startPos);
					$endPos   = strpos($location, PSX_Http::$newLine);

					if($endPos !== false)
					{
						$location = trim(substr($location, 0, $endPos));

						if($location != $request->getUrl()->__toString())
						{
							$request->setUrl(new PSX_Url($location));

							if($request->getMaxRedirects() > $count)
							{
								return $this->request($request, ++$count);
							}
							else
							{
								throw new PSX_Http_Exception('Max redirection reached');
							}
						}
					}
				}
			}
			else
			{
				if($response === false)
				{
					// if the response is false max redirection is reached
					throw new PSX_Http_Exception('Max redirection reached');
				}
				else
				{
					// if follow location is true all headers from each request are
					// included in the response but we only want return the last
					// headers
					$pos = strrpos($response, PSX_Http::$newLine . 'HTTP/');

					if($pos !== false)
					{
						$response = substr($response, $pos + strlen(PSX_Http::$newLine));
					}
				}
			}
		}


		return $response;
	}

	public function getLastError()
	{
		return $this->lastError;
	}

	public function getRequest()
	{
		return $this->request;
	}

	public function getResponse()
	{
		return $this->response;
	}
}
