<?php
/*
 *  $Id: Request.php 335 2011-10-10 14:56:24Z k42b3.x@googlemail.com $
 *
 * psx
 * A object oriented and modular based PHP framework for developing
 * dynamic web applications. For the current version and informations
 * visit <http://phpsx.org>
 *
 * Copyright (c) 2011 Christoph Kappestein <k42b3.x@gmail.com>
 *
 * This file is part of psx. psx is free software: you can
 * redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or any later version.
 *
 * psx is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with psx. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * PSX_Http_Request
 *
 * @author     Christoph Kappestein <k42b3.x@gmail.com>
 * @license    http://www.gnu.org/licenses/gpl.html GPLv3
 * @link       http://phpsx.org
 * @category   PSX
 * @package    PSX_Http
 * @version    $Revision: 335 $
 */
class PSX_Http_Request extends PSX_Http_Message
{
	protected $url;
	protected $method;
	protected $scheme;

	protected $ssl = false;
	protected $timeout;
	protected $callback;

	protected $followLocation = false;
	protected $maxRedirects   = 8;

	public function __construct(PSX_Url $url, $method, array $header = array(), $body = null, $scheme = 'HTTP/1.1')
	{
		parent::__construct($header, $body);

		$this->setUrl($url);
		$this->setMethod($method);
		$this->setScheme($scheme);

		if($url->getScheme() == 'https')
		{
			$this->setSSL(true);
		}
	}

	public function setUrl(PSX_Url $url)
	{
		$this->url = $url;

		$this->addHeader('Host', $url->getHost());
	}

	public function setMethod($method)
	{
		$this->method = $method;
	}

	public function setScheme($scheme)
	{
		$this->scheme = $scheme;
	}

	public function setSSL($ssl)
	{
		$this->ssl = (boolean) $ssl;
	}

	/**
	 * setTimeout
	 *
	 * Timeout in seconds
	 *
	 * @return void
	 */
	public function setTimeout($timeout)
	{
		$this->timeout = (integer) $timeout;
	}

	public function setCallback($callback)
	{
		$this->callback;
	}

	public function getUrl()
	{
		return $this->url;
	}

	public function getMethod()
	{
		return $this->method;
	}

	public function getScheme()
	{
		return $this->scheme;
	}

	public function isSSL()
	{
		return $this->ssl;
	}

	public function getTimeout()
	{
		return $this->timeout;
	}

	public function getCallback()
	{
		return $this->callback;
	}

	public function getLine()
	{
		$path = $this->getUrl()->getPath();
		$path = empty($path) ? '/' : $path;

		return $this->getMethod() . ' ' . $path . ' ' . $this->getScheme();
	}

	public function addHeader($key, $value)
	{
		$this->header[strtolower($key)] = $value;
	}

	public function setFollowLocation($location, $maxRedirects = 8)
	{
		$this->followLocation = (boolean) $location;
		$this->maxRedirects   = (integer) $maxRedirects;
	}

	public function getFollowLocation()
	{
		return $this->followLocation;
	}

	public function getMaxRedirects()
	{
		return $this->maxRedirects;
	}

	public function toString()
	{
		$request = $this->getLine() . PSX_Http::$newLine;

		foreach($this->header as $k => $v)
		{
			$request.= $k . ': ' . $v . PSX_Http::$newLine;
		}

		$request.= PSX_Http::$newLine;
		$request.= $this->getBody();

		return $request;
	}

	public function __toString()
	{
		return $this->toString();
	}

	public static function mergeHeader(array $defaultHeader, array $costumHeader)
	{
		$costumHeader  = array_change_key_case($costumHeader);
		$defaultHeader = array_change_key_case($defaultHeader);

		return array_merge($defaultHeader, $costumHeader);
	}
}

