<?php
/*
 *  $Id: Response.php 253 2011-03-26 13:07:02Z k42b3.x $
 *
 * psx
 * A object oriented and modular based PHP framework for developing
 * dynamic web applications. For the current version and informations
 * visit <http://phpsx.org>
 *
 * Copyright (c) 2011 Christoph Kappestein <k42b3.x@gmail.com>
 *
 * This file is part of psx. psx is free software: you can
 * redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or any later version.
 *
 * psx is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with psx. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * PSX_Http_Response
 *
 * @author     Christoph Kappestein <k42b3.x@gmail.com>
 * @license    http://www.gnu.org/licenses/gpl.html GPLv3
 * @link       http://phpsx.org
 * @category   PSX
 * @package    PSX_Http
 * @version    $Revision: 253 $
 */
class PSX_Http_Response extends PSX_Http_Message
{
	private $scheme;
	private $code;
	private $message;

	public function __construct($scheme = null, $code = null, $message = null, array $header = array(), $body = null)
	{
		parent::__construct($header, $body);

		$this->setScheme($scheme);
		$this->setCode($code);
		$this->setMessage($message);
	}

	public function setScheme($scheme)
	{
		$this->scheme = $scheme;
	}

	public function setCode($code)
	{
		$this->code = $code;
	}

	public function setMessage($message)
	{
		$this->message = $message;
	}

	public function getScheme()
	{
		return $this->scheme;
	}

	public function getCode()
	{
		return $this->code;
	}

	public function getMessage()
	{
		return $this->message;
	}

	public static function convert($request)
	{
		$response = new self();


		list($scheme, $code, $message) = self::getStatus($request);

		$response->setScheme($scheme);

		$response->setCode($code);

		$response->setMessage($message);


		list($header, $body) = self::splitResponse($request);

		$response->setHeader(self::headerToArray($header));

		$response->setBody($body);


		return $response;
	}

	private static function getStatus($response)
	{
		$line = self::getStatusLine($response);

		if($line !== false)
		{
			$parts = explode(' ', $line, 3);

			if(isset($parts[0]) && isset($parts[1]) && isset($parts[2]))
			{
				$scheme  = strval($parts[0]);
				$code    = intval($parts[1]);
				$message = strval($parts[2]);

				return array($scheme, $code, $message);
			}
			else
			{
				throw new PSX_Http_Exception('Invalid status line format');
			}
		}
		else
		{
			throw new PSX_Http_Exception('Couldnt find status line');
		}
	}

	private static function getStatusLine($response)
	{
		$pos = strpos($response, PSX_Http::$newLine);

		if($pos !== false)
		{
			return substr($response, 0, $pos);
		}
		else
		{
			return false;
		}
	}

	private static function splitResponse($response)
	{
		$pos    = strpos($response, PSX_Http::$newLine . PSX_Http::$newLine);
		$header = substr($response, 0, $pos);
		$body   = trim(substr($response, $pos + 1));

		return array($header, $body);
	}

	public static function headerToArray($header)
	{
		$lines  = explode(PSX_Http::$newLine, $header);
		$header = array();

		foreach($lines as $line)
		{
			$parts = explode(':', $line, 2);

			if(isset($parts[0]) && isset($parts[1]))
			{
				$key   = strtolower(trim($parts[0]));
				$value = trim($parts[1]);

				$header[$key] = $value;
			}
		}

		return $header;
	}
}

