<?php
/*
 *  $Id: Response.php 370 2011-11-20 18:35:59Z k42b3.x@googlemail.com $
 *
 * psx
 * A object oriented and modular based PHP framework for developing
 * dynamic web applications. For the current version and informations
 * visit <http://phpsx.org>
 *
 * Copyright (c) 2011 Christoph Kappestein <k42b3.x@gmail.com>
 *
 * This file is part of psx. psx is free software: you can
 * redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or any later version.
 *
 * psx is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with psx. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * PSX_Oauth_Provider_Data_Response
 *
 * @author     Christoph Kappestein <k42b3.x@gmail.com>
 * @license    http://www.gnu.org/licenses/gpl.html GPLv3
 * @link       http://phpsx.org
 * @category   PSX
 * @package    PSX_Oauth
 * @version    $Revision: 370 $
 */
class PSX_Oauth_Provider_Data_Response extends PSX_Data_RecordAbstract
{
	public $token;
	public $tokenSecret;
	public $params = array();

	public function getName()
	{
		return 'response';
	}

	public function getFields()
	{
		return array(

			'oauth_token'        => $this->token,
			'oauth_token_secret' => $this->tokenSecret,

		) + $this->params;
	}

	public function setToken($token)
	{
		$this->token = $token;
	}

	public function setTokenSecret($tokenSecret)
	{
		$this->tokenSecret = $tokenSecret;
	}

	public function setParams(array $params)
	{
		$this->params = $params;
	}

	public function getToken()
	{
		return $this->token;
	}

	public function getTokenSecret()
	{
		return $this->tokenSecret;
	}

	public function getParams()
	{
		return $this->params;
	}

	public function addParam($key, $value)
	{
		$this->params[$key] = $value;
	}

	public function import(PSX_Data_ReaderResult $result)
	{
		switch($result->getType())
		{
			case PSX_Data_ReaderInterface::FORM:

				$data = $result->getData();

				foreach($data as $k => $v)
				{
					switch($k)
					{
						case 'oauth_token':

							$this->setToken($v);

							break;

						case 'oauth_token_secret':

							$this->setTokenSecret($v);

							break;

						default:

							$this->addParam($k, $v);

							break;
					}
				}

				break;

			default:

				throw new PSX_Data_Exception('Can only import results from reader form');

				break;
		}
	}
}

