<?php
/*
 *  $Id: Sql.php 245 2011-03-24 01:07:52Z k42b3.x $
 *
 * psx
 * A object oriented and modular based PHP framework for developing
 * dynamic web applications. For the current version and informations
 * visit <http://phpsx.org>
 *
 * Copyright (c) 2011 Christoph Kappestein <k42b3.x@gmail.com>
 *
 * This file is part of psx. psx is free software: you can
 * redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or any later version.
 *
 * psx is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with psx. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * PSX_Session_Handler_Sql
 *
 * @author     Christoph Kappestein <k42b3.x@gmail.com>
 * @license    http://www.gnu.org/licenses/gpl.html GPLv3
 * @link       http://phpsx.org
 * @category   PSX
 * @package    PSX_Session
 * @version    $Revision: 245 $
 */
class PSX_Session_Handler_Sql implements PSX_Session_HandlerInterface
{
	private $sql;
	private $table;

	public function __construct(PSX_Sql $sql, $table)
	{
		$this->sql   = $sql;
		$this->table = $table;
	}

	public function open($path, $name)
	{
		return true;
	}

	public function close()
	{
		return true;
	}

	public function read($id)
	{
		$con  = new PSX_Sql_Condition(array('id', '=', $id));
		$data = $this->sql->select($this->table, array('data'), $con, psx_sql::SELECT_FIELD);

		return $data;
	}

	public function write($id, $data)
	{
		$this->sql->insert($this->table, array(

			'id'   => $id,
			'data' => $data,
			'date' => time(),

		));
	}

	public function delete($id)
	{
		$con = new PSX_Sql_Condition(array('id', '=', $id));

		$this->sql->delete($this->table, $con);
	}

	public function gc($max_time)
	{
		$result = $this->sql->select($this->table, array('id', 'date'), $con);

		foreach($result as $row)
		{
			if(($row['date'] + $max_time) < time())
			{
				$con = new psx_sql_condition(array('id', '=', $row['id']));

				$this->sql->delete($this->table, $con);
			}
		}

		return true;
	}
}

