<?php
/*
 *  $Id: Condition.php 358 2011-11-05 20:47:02Z k42b3.x@googlemail.com $
 *
 * psx
 * A object oriented and modular based PHP framework for developing
 * dynamic web applications. For the current version and informations
 * visit <http://phpsx.org>
 *
 * Copyright (c) 2011 Christoph Kappestein <k42b3.x@gmail.com>
 *
 * This file is part of psx. psx is free software: you can
 * redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or any later version.
 *
 * psx is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with psx. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * PSX_Sql_Condition
 *
 * @author     Christoph Kappestein <k42b3.x@gmail.com>
 * @license    http://www.gnu.org/licenses/gpl.html GPLv3
 * @link       http://phpsx.org
 * @category   PSX
 * @package    PSX_Sql
 * @version    $Revision: 358 $
 */
class PSX_Sql_Condition
{
	private static $a_op = array('=', 'IS', '!=', 'IS NOT', 'LIKE', 'NOT LIKE', '<', '>', '<=', '>=', 'IN');
	private static $l_op = array('AND', 'OR', '&&', '||');

	private $columns     = array();
	private $operators   = array();
	private $values      = array();
	private $conjunction = array();

	private $stmt;
	private $str;

	public function __construct(array $con_1 = array(), array $con_2 = array(), array $con_3 = array())
	{
		if(count($con_1) >= 3)
		{
			if(isset($con_1[3]))
			{
				$this->add($con_1[0], $con_1[1], $con_1[2], $con_1[3]);
			}
			else
			{
				$this->add($con_1[0], $con_1[1], $con_1[2]);
			}
		}

		if(count($con_2) >= 3)
		{
			if(isset($con_2[3]))
			{
				$this->add($con_2[0], $con_2[1], $con_2[2], $con_2[3]);
			}
			else
			{
				$this->add($con_2[0], $con_2[1], $con_2[2]);
			}
		}

		if(count($con_3) >= 3)
		{
			if(isset($con_1[3]))
			{
				$this->add($con_3[0], $con_3[1], $con_3[2], $con_3[3]);
			}
			else
			{
				$this->add($con_3[0], $con_3[1], $con_3[2]);
			}
		}
	}

	public function add($column, $operator, $value, $conjunction = 'AND')
	{
		if(!in_array($operator, self::$a_op))
		{
			throw new PSX_Sql_Condition_Exception('Invalid arithmetic operator (allowed: ' . implode(', ', self::$a_op) . ')');
		}

		if(!in_array($conjunction, self::$l_op))
		{
			throw new PSX_Sql_Condition_Exception('Invalid logic operator (allowed: ' . implode(', ', self::$l_op) . ')');
		}

		$this->columns[]     = $column;
		$this->operators[]   = $operator;
		$this->values[]      = $value;
		$this->conjunction[] = $conjunction;
	}

	public function getStatment()
	{
		if($this->stmt === null)
		{
			if(!empty($this->columns))
			{
				$len        = count($this->columns);
				$conditions = '';

				foreach($this->columns as $i => $column)
				{
					if(is_array($this->values[$i]))
					{
						$conditions.= $column . ' ' . $this->operators[$i] . ' (' . current($this->values[$i]) . ')';

						unset($this->values[$i]);
					}
					else
					{
						$conditions.= $column . ' ' . $this->operators[$i] . ' ?';
					}

					$conditions.= ($i < $len - 1) ? ' ' . $this->conjunction[$i] . ' ' : '';
				}

				return $this->stmt = 'WHERE ' . $conditions;
			}
			else
			{
				return $this->stmt = '';
			}
		}
		else
		{
			return $this->stmt;
		}
	}

	public function getConditions()
	{
		$conditions = array();

		foreach($this->columns as $k => $column)
		{
			$conditions[] = array($this->columns[$k], $this->operators[$k], $this->values[$k], $this->conjunction[$k]);
		}

		return $conditions;
	}

	public function getValues()
	{
		return $this->values;
	}

	public function hasCondition()
	{
		return count($this->columns) > 0;
	}

	public function clear()
	{
		$this->stmt = null;
		$this->str  = null;

		$this->columns     = array();
		$this->operators   = array();
		$this->values      = array();
		$this->conjunction = array();
	}

	public function merge(PSX_Sql_Condition $condition)
	{
		$conditions = $condition->getConditions();

		foreach($conditions as $k => $con)
		{
			$this->columns[]     = $con[0];
			$this->operators[]   = $con[1];
			$this->values[]      = $con[2];
			$this->conjunction[] = $con[3];
		}
	}

	public function toString()
	{
		if($this->str === null)
		{
			if(!empty($this->columns))
			{
				$len        = count($this->columns);
				$conditions = '';

				foreach($this->columns as $i => $column)
				{
					if(is_int($this->values[$i]) || is_float($this->values[$i]) || is_bool($this->values[$i]))
					{
						$value = $this->values[$i];
					}
					elseif(is_array($this->values[$i]))
					{
						$value = '(' . current($this->values[$i]) . ')';
					}
					else
					{
						$value = '"' . mysql_real_escape_string($this->values[$i]) . '"';
					}

					$conditions.= $column . ' ' . $this->operators[$i] . ' ' . $value;

					$conditions.= ($i < $len - 1) ? ' ' . $this->conjunction[$i] . ' ' : '';
				}

				return $this->str = 'WHERE ' . $conditions;
			}
			else
			{
				return $this->str = '';
			}
		}
		else
		{
			return $this->str;
		}
	}

	public function __tostring()
	{
		return $this->toString();
	}

	public static function parse($stmt)
	{
		$condition = new self();

		do
		{
			$column      = self::getColumn($stmt);
			$operator    = self::getOperator($stmt);
			$value       = self::getValue($stmt);
			$conjunction = self::getConjunction($stmt);

			if(!empty($conjunction))
			{
				$condition->add($column, $operator, $value, $conjunction);
			}
			else
			{
				$condition->add($column, $operator, $value);
			}
		}
		while(!empty($column) && !empty($operator) && !empty($value));

		return $condition;
	}

	private static function getColumn(&$str)
	{
		$str = trim($str);
		$pos = strpos($str, ' ');

		if($pos !== false)
		{
			$column = substr($str, 0, $pos);
			$str    = substr($str, $pos);

			return $column;
		}
		else
		{
			$column = $str;
			$str    = '';

			return $column;
		}
	}

	private static function getOperator(&$str)
	{
		$str = trim($str);

		foreach(self::$a_op as $op)
		{
			$len = strlen($op);

			if(strcasecmp(substr($str, 0, $len), $op) == 0)
			{
				$operator = substr($str, 0, $len);
				$str      = substr($str, $len);

				return $operator;
			}
		}

		return false;
	}

	private static function getValue(&$str)
	{
		$str = trim($str);

		if(empty($str))
		{
			return false;
		}

		if($str[0] == '"')
		{
			$pos = strpos($str, '"', 1);

			if($pos !== false)
			{
				$value = substr($str, 1, $pos);
				$str   = substr($str, $pos);

				return $value;
			}
		}
		elseif($str[0] == '\'')
		{
			$pos = strpos($str, '\'', 1);

			if($pos !== false)
			{
				$value = substr($str, 1, $pos);
				$str   = substr($str, $pos);

				return $value;
			}
		}
		else
		{
			$len = strlen($str);

			for($i = 0; $i < $len; $i++)
			{
				if(!trim($str[$i]))
				{
					$value = substr($str, 0, $i);
					$str   = substr($str, $i);

					return $value;
				}
			}

			$value = $str;
			$str   = '';

			return $value;
		}
	}

	private static function getConjunction(&$str)
	{
		$str = trim($str);

		foreach(self::$l_op as $op)
		{
			$len = strlen($op);

			if(strcasecmp(substr($str, 0, $len), $op) == 0)
			{
				$conjunction = substr($str, 0, $len);
				$str         = substr($str, $len);

				return $conjunction;
			}
		}

		return false;
	}
}