<?php
/*
 *  $Id: Stmt.php 245 2011-03-24 01:07:52Z k42b3.x $
 *
 * psx
 * A object oriented and modular based PHP framework for developing
 * dynamic web applications. For the current version and informations
 * visit <http://phpsx.org>
 *
 * Copyright (c) 2011 Christoph Kappestein <k42b3.x@gmail.com>
 *
 * This file is part of psx. psx is free software: you can
 * redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or any later version.
 *
 * psx is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with psx. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * PSX_Sql_Driver_Mysql_Stmt
 *
 * @author     Christoph Kappestein <k42b3.x@gmail.com>
 * @license    http://www.gnu.org/licenses/gpl.html GPLv3
 * @link       http://phpsx.org
 * @category   PSX
 * @package    PSX_Sql
 * @version    $Revision: 245 $
 */
class PSX_Sql_Driver_Mysql_Stmt implements PSX_Sql_StmtInterface
{
	const STRING = 0x0;
	const INT    = 0x1;
	const BOOL   = 0x2;
	const FLOAT  = 0x3;

	private $handle;
	private $sql;
	private $result;
	private $params = array();

	public function __construct($connection, $sql)
	{
		$this->handle = $connection;
		$this->sql    = $sql;
	}

	public function bindParam($value)
	{
		array_push($this->params, array(

			'type'  => $this->getType($value),
			'value' => $value,

		));
	}

	public function execute()
	{
		$sql = $this->sql;

		if(count($this->params) > 0)
		{
			foreach($this->params as $param)
			{
				if($param['type'] == self::STRING)
				{
					$sql = $this->sqlReplace($sql, '"' . mysql_real_escape_string($param['value']) . '"');
				}
				else
				{
					$sql = $this->sqlReplace($sql, mysql_real_escape_string($param['value']));
				}
			}

			$this->params = array();
		}

		$this->result = mysql_query($sql, $this->handle);
	}

	public function numRows()
	{
		if(stristr($this->sql, 'SELECT') !== false)
		{
			return mysql_num_rows($this->result);
		}
		else
		{
			return mysql_affected_rows($this->handle);
		}
	}

	public function fetchAssoc()
	{
		$result = array();

		while($row = mysql_fetch_assoc($this->result))
		{
			$result[] = $row;
		}

		mysql_free_result($this->result);

		return $result;
	}

	public function fetchObject($class = 'stdClass', array $args = array())
	{
		$result = array();

		$reflection  = new ReflectionClass($class);
		$constructor = $reflection->getConstructor();

		if(empty($constructor))
		{
			while($row = mysql_fetch_assoc($this->result))
			{
				$obj = $reflection->newInstance();

				foreach($row as $key => $val)
				{
					$obj->$key = $val;
				}

				$result[] = $obj;
			}
		}
		else
		{
			while($row = mysql_fetch_assoc($this->result))
			{
				$obj = $reflection->newInstanceArgs($args);

				foreach($row as $key => $val)
				{
					$obj->$key = $val;
				}

				$result[] = $obj;
			}
		}

		mysql_free_result($this->result);

		return $result;
	}

	public function getHandle()
	{
		return $this->result;
	}

	public function close()
	{
		$this->sql = null;
	}

	public function error()
	{
		return mysql_error($this->handle);
	}

	private function getType($var)
	{
		if(is_string($var))
		{
			return self::STRING;
		}
		else if(is_int($var))
		{
			return self::INT;
		}
		else if(is_bool($var))
		{
			return self::BOOL;
		}
		else if(is_float($var))
		{
			return self::FLOAT;
		}

		return self::STRING;
	}

	private function sqlReplace($sql, $value)
	{
		$pos = strpos($sql, '?');
		$f   = substr($sql, 0, $pos);
		$b   = substr($sql, $pos + 1);

		return $f . $value . $b;
	}
}
