<?php
/*
 *  $Id: Upload.php 354 2011-10-29 15:25:33Z k42b3.x@googlemail.com $
 *
 * psx
 * A object oriented and modular based PHP framework for developing
 * dynamic web applications. For the current version and informations
 * visit <http://phpsx.org>
 *
 * Copyright (c) 2011 Christoph Kappestein <k42b3.x@gmail.com>
 *
 * This file is part of psx. psx is free software: you can
 * redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or any later version.
 *
 * psx is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with psx. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * PSX_Upload
 *
 * A class to get the $_FILES variables and upload a file. See the
 * PSX_InputAbstract class how you can access values and use the PSX_Validate
 * class. Here a short example how to upload a file.
 * <code>
 * $upload = new PSX_Upload(new PSX_Validate());
 * $file   = $upload->userfile(null, array(new PSX_Filter_Length(3, 1024)));
 *
 * if($file !== false && $file->move('/home/foo/upload/' . $file->getName()))
 * {
 *   // file successful uploaded
 * }
 * else
 * {
 *   // an error occured
 * }
 * </code>
 *
 * In this exmaple the file must have a length between 3 and 1024.
 *
 * @author     Christoph Kappestein <k42b3.x@gmail.com>
 * @license    http://www.gnu.org/licenses/gpl.html GPLv3
 * @link       http://phpsx.org
 * @category   PSX
 * @package    PSX_Upload
 * @version    $Revision: 354 $
 */
class PSX_Upload extends PSX_InputAbstract
{
	public function has($key)
	{
		return isset($_FILES[$key]);
	}

	public function get($key)
	{
		if($this->has($key))
		{
			return new PSX_Upload_File($_FILES[$key]);
		}
		else
		{
			return false;
		}
	}

	public function set($key, $value)
	{
		$_FILES[$key] = $value;
	}

	public function delete($key)
	{
		if($this->has($key))
		{
			unset($_FILES[$key]);
		}
	}

	public function getContainer()
	{
		return $_FILES;
	}
}
