<?php
/*
 *  $Id: Numerative.php 245 2011-03-24 01:07:52Z k42b3.x $
 *
 * psx
 * A object oriented and modular based PHP framework for developing
 * dynamic web applications. For the current version and informations
 * visit <http://phpsx.org>
 *
 * Copyright (c) 2011 Christoph Kappestein <k42b3.x@gmail.com>
 *
 * This file is part of psx. psx is free software: you can
 * redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or any later version.
 *
 * psx is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with psx. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * PSX_Util_Numerative
 *
 * @author     Christoph Kappestein <k42b3.x@gmail.com>
 * @license    http://www.gnu.org/licenses/gpl.html GPLv3
 * @link       http://phpsx.org
 * @category   PSX
 * @package    PSX_Util
 * @version    $Revision: 245 $
 */
class PSX_Util_Numerative
{
	public static $systems = array(

		'bin' => array(0 => '0', 1 => '1'),
		'oct' => array(0 => '0', 1 => '1', 2 => '2', 3 => '3', 4 => '4', 5 => '5', 6 => '6', 7 => '7'),
		'dez' => array(0 => '0', 1 => '1', 2 => '2', 3 => '3', 4 => '4', 5 => '5', 6 => '6', 7 => '7', 8 => '8', 9 => '9'),
		'hex' => array(0 => '0', 1 => '1', 2 => '2', 3 => '3', 4 => '4', 5 => '5', 6 => '6', 7 => '7', 8 => '8', 9 => '9', 10 => 'A', 11 => 'B', 12 => 'C', 13 => 'D', 14 => 'E', 15 => 'F'),

	);

	public static function bin2oct($bin)
	{
		return self::dez_to_x('oct', self::x_to_dez('bin', $bin));
	}

	public static function bin2dez($bin)
	{
		return self::dez_to_x('dez', self::x_to_dez('bin', $bin));
	}

	public static function bin2hex($bin)
	{
		return self::dez_to_x('hex', self::x_to_dez('bin', $bin));
	}

	public static function oct2bin($oct)
	{
		return self::dez_to_x('bin', self::x_to_dez('oct', $oct));
	}

	public static function oct2dez($oct)
	{
		return self::dez_to_x('dez', self::x_to_dez('oct', $oct));
	}

	public static function oct2hex($oct)
	{
		return self::dez_to_x('hex', self::x_to_dez('oct', $oct));
	}

	public static function dez2bin($dez)
	{
		return self::dez_to_x('bin', $dez);
	}

	public static function dez2oct($dez)
	{
		return self::dez_to_x('oct', $dez);
	}

	public static function dez2hex($dez)
	{
		return self::dez_to_x('hex', $dez);
	}

	public static function hex2bin($hex)
	{
		return self::dez_to_x('bin', self::x_to_dez('hex', $hex));
	}

	public static function hex2oct($hex)
	{
		return self::dez_to_x('oct', self::x_to_dez('hex', $hex));
	}

	public static function hex2dez($hex)
	{
		return self::dez_to_x('dez', self::x_to_dez('hex', $hex));
	}

	public static function x_to_dez($system, $x)
	{
		if(!isset(self::$systems[$system]))
		{
			throw new numerative_exception('invalid numerative system');
		}
		else
		{
			$d = '';
			$n = array_flip(self::$systems[$system]);
			$x = strrev(strval($x));
			$c = 1;

			for($i = 0; $i < strlen($x); $i++)
			{
				$d+= $n[$x{$i}] * $c;

				$c*= count($n);
			}

			return $d;
		}
	}

	public static function dez_to_x($system, $dez)
	{
		if(!isset(self::$systems[$system]))
		{
			throw new numerative_exception('invalid numerative system');
		}
		else
		{
			$n = self::$systems[$system];
			$d = intval($dez);
			$b = count($n);
			$x = '';

			if($d == 0)
			{
				return $n[0];
			}

			if($d < 0)
			{
				throw new numerative_exception('cant convert negative numbers because we have no sign to display negative values');
			}

			while($d > 0)
			{
				$x.= $n[intval($d % $b)];
				$d = intval($d / $b);
			}

			return strrev($x);
		}
	}
}