<?php
/*
 *  $Id: Webfinger.php 349 2011-10-23 14:32:42Z k42b3.x@googlemail.com $
 *
 * psx
 * A object oriented and modular based PHP framework for developing
 * dynamic web applications. For the current version and informations
 * visit <http://phpsx.org>
 *
 * Copyright (c) 2011 Christoph Kappestein <k42b3.x@gmail.com>
 *
 * This file is part of psx. psx is free software: you can
 * redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or any later version.
 *
 * psx is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with psx. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * PSX_Webfinger
 *
 * @author     Christoph Kappestein <k42b3.x@gmail.com>
 * @license    http://www.gnu.org/licenses/gpl.html GPLv3
 * @link       http://phpsx.org
 * @category   PSX
 * @package    PSX_Webfinger
 * @version    $Revision: 349 $
 */
class PSX_Webfinger
{
	private $http;

	public function __construct(PSX_Http $http)
	{
		$this->http = $http;
	}

	public function getHostMetaXrd($url)
	{
		$url = rtrim($url, '/') . '/.well-known/host-meta';
		$url = new PSX_Url($url);

		$request  = new PSX_Http_GetRequest($url, array('Accept' => 'application/xrd+xml'));
		$request->setFollowLocation(true);
		$response = $this->http->request($request);

		if($response->getCode() == 200)
		{
			$xrd = simplexml_load_string($response->getBody());

			// validate host
			$host = (string) $xrd->children('http://host-meta.net/xrd/1.0');

			if(strcmp($host, $url->getHost()) !== 0)
			{
				throw new PSX_Webfinger_Exception('Invalid host');
			}

			return $xrd;
		}
		else
		{
			throw new PSX_Webfinger_Exception('Invalid response code ' . $response->getCode());
		}
	}

	public function getLrddTemplate($url)
	{
		$xrd = $this->getHostMetaXrd($url);

		if(isset($xrd->Link))
		{
			$template = null;

			foreach($xrd->Link as $link)
			{
				if(isset($link['rel']) && isset($link['template']) && strcasecmp($link['rel'], 'lrdd') == 0)
				{
					if(strpos($link['template'], '{uri}') !== false)
					{
						$template = $link['template'];

						break;
					}
					else
					{
						throw new PSX_Webfinger_Exception('Template link does not contain {uri}');
					}
				}
			}

			if($template !== null)
			{
				return $template;
			}
			else
			{
				throw new PSX_Webfinger_Exception('Found no link lrdd element');
			}
		}
		else
		{
			throw new PSX_Webfinger_Exception('Invalid XRD received');
		}
	}

	public function getLrddXrd($uri, $lrddTemplate)
	{
		$lrddUrl  = new PSX_Url(str_replace('{uri}', urlencode($uri), $lrddTemplate));
		$request  = new PSX_Http_GetRequest($lrddUrl, array('Accept' => 'application/xml'));
		$request->setFollowLocation(true);
		$response = $this->http->request($request);

		if($response->getCode() == 200)
		{
			$xrd = simplexml_load_string($response->getBody());

			if(isset($xrd->success) && strcasecmp($xrd->success, 'false') == 0)
			{
				$msg = isset($xrd->text) ? $xrd->text : null;

				if(!empty($msg))
				{
					throw new PSX_Webfinger_Exception($msg);
				}
				else
				{
					throw new PSX_Webfinger_Exception('Could not discover xrd');
				}
			}

			return $xrd;
		}
		else
		{
			throw new PSX_Webfinger_Exception('Invalid response code ' . $response->getCode());
		}
	}

	public function getAcctProfile($email, $lrddTemplate)
	{
		$acct = 'acct:' . $email;
		$xrd  = $this->getLrddXrd($acct, $lrddTemplate);

		if(isset($xrd->Subject) && isset($xrd->Link))
		{
			if(strcmp($xrd->Subject, $acct) !== 0)
			{
				throw new PSX_Webfinger_Exception('Invalid subject');
			}

			foreach($xrd->Link as $link)
			{
				if(isset($link['rel']) && $link['rel'] == 'profile')
				{
					return (string) $link['href'];

					break;
				}
			}

			throw new PSX_Webfinger_Exception('Could not find profile');
		}
		else
		{
			throw new PSX_Webfinger_Exception('Could not discover account');
		}
	}
}

