<?php
/*
 *  $Id: oembed.php 254 2011-05-18 18:18:20Z k42b3.x $
 *
 * amun
 * A social content managment system based on the psx framework. For
 * the current version and informations visit <http://amun.phpsx.org>
 *
 * Copyright (c) 2010 Christoph Kappestein <k42b3.x@gmail.com>
 *
 * This file is part of amun. amun is free software: you can
 * redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or any later version.
 *
 * amun is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with amun. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * oembed
 *
 * @author     Christoph Kappestein <k42b3.x@gmail.com>
 * @license    http://www.gnu.org/licenses/gpl.html GPLv3
 * @link       http://amun.phpsx.org
 * @category   module
 * @package    api
 * @subpackage content_media
 * @version    $Revision: 254 $
 */
class oembed extends Amun_ApiAbstract
{
	public function onGet()
	{
		if($this->user->hasRight('content_media_view'))
		{
			try
			{
				$url = $this->get->url('string', array(new PSX_Filter_UrlDecode(), new PSX_Filter_Length(7, 256), new PSX_Filter_Url()));

				if(!$this->filter->hasError())
				{
					$url = new PSX_Url($url);
				}
				else
				{
					throw new PSX_Data_Exception('Invalid url');
				}


				$servlet = new Amun_Content_Media_Servlet($this->config, $this->sql, $this->vars);

				$params = $this->getRequestParams();

				$fields = !empty($params['fields']) ? $params['fields'] : array('id', 'authorName', 'authorProfileUrl', 'title', 'path', 'type', 'size', 'mimeType', 'url', 'date');

				$servlet->setFields($fields);

				$servlet->addSelectCondition('id', '=', $url->getParam('id'));

				$resultset = $servlet->getAll($params['startIndex'], $params['count'], $params['sortBy'], $params['sortOrder'], $params['filterBy'], $params['filterOp'], $params['filterValue'], $params['updatedSince']);


				if($resultset->hasResults())
				{
					$entry    = $resultset->current();
					$response = new PSX_Data_Record();

					$response->version       = '1.0';
					$response->title         = $entry['title'];
					$response->author_name   = $entry['authorName'];
					$response->author_url    = $entry['authorProfileUrl'];
					$response->provider_name = 'amun (' . $this->config['amun_version'] . ')';
					$response->provider_url  = $this->config['psx_url'];


					// check type
					if($entry['type'] != 'video' && $entry['type'] != 'audio')
					{
						throw new PSX_Data_Exception('Invalid media type');
					}


					// get size
					$size = getimagesize($this->vars['core.media_path'] . '/' $entry['authorName'] . '/' . $entry['path'])

					if($size === false)
					{
						throw new PSX_Data_Exception('Invalid media');
					}

					list($width, $height, $type, $attr) = $size;


					// url
					$url = $this->config['psx_url'] . '/' . $this->config['psx_dispatch'] . 'api/content/media/serve?id=' . $entry['id'];

					switch($entry['type'])
					{
						case 'video':

							$html = <<<HTML
<object {$attr}>
	<param name="movie" value="{$url}"></param>
	<param name="allowFullScreen" value="true"></param>
	<param name="allowscriptaccess" value="always"></param>
	<embed src="{$url}" type="application/x-shockwave-flash" {$attr} allowscriptaccess="always" allowfullscreen="true"></embed>
</object>
HTML;

							$response->type   = 'video';
							$response->html   = $html;
							$response->width  = $width;
							$response->height = $height;

							break;

						case 'image':

							$response->type   = 'photo';
							$response->url    = $url;
							$response->width  = $width;
							$response->height = $height;

							break;
					}

					$this->setResponse($response);
				}
				else
				{
					throw new PSX_Data_Exception('Url doesent exist');
				}
			}
			catch(Exception $e)
			{
				$msg = new PSX_Data_Message($e->getMessage(), false);

				$this->setResponse($msg);
			}
		}
		else
		{
			$msg = new PSX_Data_Message('Access not allowed', false);

			$this->setResponse($msg, $this->user->isAnonymous() ? 401 : 403);
		}
	}
}

