<?php
/*
 *  $Id: connect.php 431 2011-12-03 18:11:04Z k42b3.x@googlemail.com $
 *
 * amun
 * A social content managment system based on the psx framework. For
 * the current version and informations visit <http://amun.phpsx.org>
 *
 * Copyright (c) 2010 Christoph Kappestein <k42b3.x@gmail.com>
 *
 * This file is part of amun. amun is free software: you can
 * redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or any later version.
 *
 * amun is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with amun. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * connect
 *
 * @author     Christoph Kappestein <k42b3.x@gmail.com>
 * @license    http://www.gnu.org/licenses/gpl.html GPLv3
 * @link       http://amun.phpsx.org
 * @category   module
 * @package    my
 * @version    $Revision: 431 $
 */
class connect extends Amun_ApplicationAbstract
{
	private $requestId;
	private $claimedId;
	private $identity;
	private $returnTo;
	private $sreg;
	private $oauth;

	public function onLoad()
	{
		if($this->user->hasRight('service_my_view'))
		{
			// add path
			$this->path->add('Connect', $this->page->url . '/connect');


			// check whether connection was started
			if(is_array($_SESSION['openid_connect']))
			{
				$params = $_SESSION['openid_connect'];

				$this->requestId = isset($params['requestId']) ? $params['requestId'] : null;
				$this->claimedId = isset($params['claimedId']) ? $params['claimedId'] : null;
				$this->identity  = isset($params['identity'])  ? $params['identity']  : null;
				$this->returnTo  = isset($params['returnTo'])  ? $params['returnTo']  : null;
				$this->sreg      = isset($params['sreg'])      ? $params['sreg']      : null;
				$this->oauth     = isset($params['oauth'])     ? $params['oauth']     : null;

				$this->template->assign('rpHost', parse_url($this->returnTo, PHP_URL_HOST));
			}
			else
			{
				throw new Amun_Exception('No connection was initialized');
			}


			// check whether access is already allowed
			$allowed = Amun_Sql_Table_Registry::get('System_Connect_Access')
				->select(array('allowed'))
				->where('userId', '=', $this->user->id)
				->where('returnTo', '=', $this->returnTo)
				->getField();

			if($allowed === '1')
			{
				$this->allowAccess();
			}

			if($allowed === '0')
			{
				$this->denyAccess();
			}


			// template
			$this->htmlCss->add('my');

			$this->template->set('application/' . $this->page->application . '/' . __CLASS__ . '.tpl');
		}
		else
		{
			throw new Amun_Exception('Access not allowed');
		}
	}

	public function onPost()
	{
		try
		{
			if(isset($_POST['allow']))
			{
				$this->allowAccess();
			}

			if(isset($_POST['deny']))
			{
				$this->denyAccess();
			}
		}
		catch(Exception $e)
		{
			// cancel request
			$url = new PSX_Url($this->returnTo);

			$url->addParam('openid.ns', PSX_OpenId_ProviderAbstract::NS);
			$url->addParam('openid.mode', 'error');
			$url->addParam('openid.error', $e->getMessage());

			header('Location: ' . strval($url));
			exit;
		}
	}

	private function allowAccess()
	{
		// insert or update access
		if(isset($_POST['remember']))
		{
			$now = new DateTime('NOW', $this->registry['core.default_timezone']);

			$this->sql->replace($this->registry['table.system_connect_access'], array(

				'userId'   => $this->user->id,
				'returnTo' => $this->returnTo,
				'allowed'  => 1,
				'date'     => $now->format(PSX_Time::SQL),

			));
		}


		// delete session
		$_SESSION['openid_connect'] = null;


		// build url
		$secret = Amun_Security::generateToken(40);

		$res = new PSX_OpenId_Provider_Data_ResRequest();

		$res->setOpEndpoint($this->config['psx_url'] . '/' . $this->config['psx_dispatch'] . 'api/service/my/signon');
		$res->setClaimedId($this->claimedId);
		$res->setIdentity($this->identity);
		$res->setReturnTo($this->returnTo);
		$res->setResponseNonce(gmdate('Y-m-d\TH:i:s\Z') . Amun_Security::generateToken(15));
		$res->setAssocHandle(Amun_Security::generateToken(40));


		// extensions
		$params = array();

		// sreg
		if(!empty($this->sreg))
		{
			$params['ns.sreg'] = PSX_OpenId_Extension_Sreg::NS;
			$params['sreg.nickname'] = $this->user->name;
		}

		// oauth
		if(!empty($this->oauth))
		{
			$consumerKey = isset($this->oauth['consumer']) ? $this->oauth['consumer'] : null;

			$row = Amun_Sql_Table_Registry::get('System_Api')
				->select(array('id', 'consumerKey'))
				->where('consumerKey', '=', $consumerKey)
				->getRow();

			if(!empty($row))
			{
				$token    = Amun_Security::generateToken(40);
				$verifier = Amun_Security::generateToken(32);
				$date     = new DateTime('NOW', $this->registry['core.default_timezone']);

				$this->sql->insert($this->registry['table.system_api_request'], array(

					'apiId'       => $row['id'],
					'userId'      => $this->user->id,
					'status'      => Amun_System_Api::APPROVED,
					'ip'          => $_SERVER['REMOTE_ADDR'],
					'nonce'       => Amun_Security::generateToken(16),
					'callback'    => 'oob',
					'token'       => $token,
					'tokenSecret' => '',
					'verifier'    => $verifier,
					'timestamp'   => time(),
					'expire'      => 'PT30M',
					'date'        => $date->format(PSX_Time::SQL),

				));


				// insert access
				$this->sql->replace($this->registry['table.system_api_access'], array(

					'apiId'   => $row['id'],
					'userId'  => $this->user->id,
					'allowed' => 1,
					'date'    => $date->format(PSX_Time::SQL),

				));


				// add params
				$params['ns.oauth'] = PSX_OpenId_Extension_Oauth::NS;
				$params['oauth.request_token'] = $token;
				$params['oauth.verifier'] = $verifier;
			}
			else
			{
				throw new Amun_Exception('Invalid consumer');
			}
		}


		// set params and build request
		$res->setParams($params);


		// insert connect
		$now = new DateTime('NOW', $this->registry['core.default_timezone']);

		$this->sql->insert($this->registry['table.system_connect'], array(

			'userId'        => $this->user->id,
			'status'        => Amun_System_Api::APPROVED,
			'claimedId'     => $this->claimedId,
			'identity'      => $this->identity,
			'returnTo'      => $this->returnTo,
			'responseNonce' => $res->getResponseNonce(),
			'assocHandle'   => $res->getAssocHandle(),
			'sig'           => $res->buildSignature($secret),
			'secret'        => $secret,
			'expire'        => 'PT1H',
			'date'          => $now->format(PSX_Time::SQL),

		));


		// redirect to rp
		$res->redirect();
	}

	private function denyAccess()
	{
		// insert or update access
		if(isset($_POST['remember']))
		{
			$now = new DateTime('NOW', $this->registry['core.default_timezone']);

			$this->sql->replace($this->registry['table.system_connect_access'], array(

				'userId'   => $this->user->id,
				'returnTo' => $this->returnTo,
				'allowed'  => 0,
				'date'     => $now->format(PSX_Time::SQL),

			));
		}


		// delete oauth token
		if(!empty($this->oauth))
		{
			$con = new PSX_Sql_Condition(array('id', '=', $this->oauth['requestId']));

			$this->sql->delete($this->registry['table.system_api_request'], $con);
		}


		// delete session
		$_SESSION['openid_connect'] = null;


		// cancel request
		$url = new PSX_Url($this->request['callback']);

		$url->addParam('openid.ns', 'http://specs.openid.net/auth/2.0');
		$url->addParam('openid.mode', 'cancel');

		header('Location: ' . strval($url));
		exit;
	}
}

