<?php
/*
 *  $Id: callback.php 431 2011-12-03 18:11:04Z k42b3.x@googlemail.com $
 *
 * amun
 * A social content managment system based on the psx framework. For
 * the current version and informations visit <http://amun.phpsx.org>
 *
 * Copyright (c) 2010 Christoph Kappestein <k42b3.x@gmail.com>
 *
 * This file is part of amun. amun is free software: you can
 * redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or any later version.
 *
 * amun is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with amun. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * callback
 *
 * @author     Christoph Kappestein <k42b3.x@gmail.com>
 * @license    http://www.gnu.org/licenses/gpl.html GPLv3
 * @link       http://amun.phpsx.org
 * @category   module
 * @package    my
 * @version    $Revision: 431 $
 */
class callback extends Amun_ApplicationAbstract
{
	public function onLoad()
	{
		try
		{
			if(!$this->session->has('openid_register_user'))
			{
				throw new Amun_Exception('Login process was not initialized');
			}


			// http
			$this->http = new PSX_Http(new PSX_Http_Handler_Curl());


			// initialize openid
			$openid = new PSX_OpenId($this->http, $this->config['psx_url']);

			if($openid->verify() === true)
			{
				$identity = $openid->getIdentifier();

				if(!empty($identity))
				{
					if($this->session->openid_register_user === true)
					{
						// check whether registration is enabled
						if(!$this->registry['my.registration_enabled'])
						{
							throw new Amun_Exception('Registration is disabled');
						}


						// get signed data
						$data = $openid->getData();


						// try to get user name
						$name = $this->getName($data);

						if(empty($name))
						{
							// probably we could use here the claimedId as
							// username but for now we allow only openid
							// registration if we have at least a username
							// $openid->getClaimedId();

							throw new Amun_Exception('No username given please use the sreg extension to provide an username');
						}

						$name = $this->normalizeName($name);


						// if an oauth extension was used exchange the token for
						// an access token
						$hostId = (integer) $this->session->openid_register_user_host_id;
						$resp   = null;

						if(!empty($hostId))
						{
							$resp = $this->getOauthAccessToken($hostId, $data);
						}


						// create user account
						$account = Amun_Sql_Table_Registry::get('User_Account')->getRecord();
						$account->setGroupId($this->registry['core.default_user_group']);
						$account->setHostId($hostId);
						$account->setStatus(Amun_User_Account::REMOTE);
						$account->setIdentity($identity);
						$account->setName($name);
						$account->setPw(Amun_Security::generatePw());

						$handler = new Amun_User_Account_Handler($this->user);

						$account = $handler->create($account);


						// insert access tokens if requested
						if($resp instanceof PSX_Oauth_Provider_Data_Response)
						{
							$date = new DateTime('NOW', $this->registry['core.default_timezone']);

							$this->sql->insert($this->registry['table.system_host_request'], array(

								'hostId'      => $hostId,
								'userId'      => $account->id,
								'ip'          => $_SERVER['REMOTE_ADDR'],
								'token'       => $resp->getToken(),
								'tokenSecret' => $resp->getTokenSecret(),
								'expire'      => 'P6M',
								'date'        => $date->format(PSX_Time::SQL),

							));
						}


						// if the id is not set the account was add to the
						// approval table
						if(isset($account->id))
						{
							$this->session->set('amun_id', $account->id);
							$this->session->set('amun_t', time());
						}
						else
						{
							throw new Amun_Exception('Could not create account');
						}
					}
					else
					{
						$identity = sha1(Amun_Security::getSalt() . $identity);

						$con = new PSX_Sql_Condition(array('identity', '=', $identity));
						$id  = $this->sql->select($this->registry['table.user_account'], array('id'), $con, PSX_Sql::SELECT_FIELD);

						if($id !== false)
						{
							$this->session->set('amun_id', $id);
							$this->session->set('amun_t', time());
						}
						else
						{
							throw new Amun_Exception('Identity doesnt exist');
						}
					}


					// redirect
					header('Location: ' . $this->config['psx_url']);
					exit;
				}
				else
				{
					throw new Amun_Exception('Invalid identity');
				}
			}
			else
			{
				throw new Amun_Exception('Authentication failed');
			}
		}
		catch(Exception $e)
		{
			$this->template->assign('error', $e->getMessage() . "\n\n" . $e->getTraceAsString());
		}


		// add path
		$this->path->add('Login', $this->page->url . '/login');
		$this->path->add('Callback', $this->page->url . '/login/callback');


		// template
		$this->htmlCss->add('my');

		$this->template->set('application/' . $this->page->application . '/login/' . __CLASS__ . '.tpl');
	}

	private function getName(array $data)
	{
		// sreg extension
		$params = PSX_OpenId_ProviderAbstract::getExtension($data, PSX_OpenId_Extension_Sreg::NS);

		if(isset($params['nickname']))
		{
			return $params['nickname'];
		}
		else if(isset($params['fullname']))
		{
			return $params['fullname'];
		}

		// ax extension
		$params = PSX_OpenId_ProviderAbstract::getExtension($data, PSX_OpenId_Extension_Ax::NS);

		$keyFn = null;
		$keyF  = null;
		$keyL  = null;

		foreach($params as $k => $v)
		{
			if($v == 'http://axschema.org/namePerson')
			{
				$keyFn = str_replace('type', 'value', $k);
			}

			if($v == 'http://axschema.org/namePerson/first')
			{
				$keyF = str_replace('type', 'value', $k);
			}

			if($v == 'http://axschema.org/namePerson/last')
			{
				$keyL = str_replace('type', 'value', $k);
			}
		}

		if(isset($params[$keyF]) && isset($params[$keyL]))
		{
			return $params[$keyF] . ' ' . $params[$keyL];
		}
		elseif(isset($params[$keyFn]))
		{
			return $params[$keyFn];
		}
	}

	private function getOauthAccessToken($hostId, array $data)
	{
		$data = PSX_OpenId_ProviderAbstract::getExtension($data, PSX_OpenId_Extension_Oauth::NS);

		$token    = isset($data['request_token']) ? $data['request_token'] : null;
		$verifier = isset($data['verifier'])      ? $data['verifier']      : null;

		if($hostId > 0 && !empty($token) && !empty($verifier))
		{
			$row = Amun_Sql_Table_Registry::get('System_Host')
				->select(array('consumerKey', 'consumerSecret', 'url'))
				->where('id', '=', $hostId)
				->where('status', '=', Amun_System_Host::NORMAL)
				->getRow();

			if(!empty($row))
			{
				$url   = $this->discoverOauthAcessUrl(new PSX_Url($row['url']));
				$oauth = new PSX_Oauth($this->http);

				return $oauth->accessToken($url, $row['consumerKey'], $row['consumerSecret'], $token, '', $verifier);
			}
			else
			{
				throw new Amun_Exception('Invalid host id');
			}
		}
	}

	public function discoverOauthAcessUrl(PSX_Url $url)
	{
		$yadis = new PSX_Yadis($this->http);
		$xrds  = $yadis->discover($url);

		if($xrds !== false && isset($xrds->service))
		{
			$uri = null;

			foreach($xrds->service as $service)
			{
				if(in_array('http://oauth.net/core/1.0/endpoint/access', $service->getType()))
				{
					$uri = $service->getUri();
				}
			}

			if(!empty($uri))
			{
				return new PSX_Url($uri);
			}
			else
			{
				throw new Amun_Exception('Could not find service');
			}
		}
		else
		{
			throw new Amun_Exception('Could not find xrds');
		}
	}

	/**
	 * normalizeName
	 *
	 * We receive an name from an OpenID provider the name is handled as an
	 * untrsuted value because of that we go through each sign and check
	 * whether it is valid. The string wich returned by this method is a
	 * valid name
	 *
	 * @param string $name
	 * @return string
	 */
	private function normalizeName($name)
	{
		$norm = '';
		$len  = strlen($name);


		// replace white space with period
		$name = str_replace(' ', '.', $name);


		// name can only contain A-Z a-z 0-9 .
		$period = false;

		for($i = 0; $i < $len; $i++)
		{
			$ascii = ord($name[$i]);

			# alpha (A - Z / a - z / 0 - 9 / .)
			if(($ascii >= 0x41 && $ascii <= 0x5A) || ($ascii >= 0x61 && $ascii <= 0x7A) || ($ascii >= 0x30 && $ascii <= 0x39))
			{
				$norm.= $name[$i];
			}

			if($period === false && $ascii == 0x2E)
			{
				$norm.= $name[$i];

				$period = true;
			}
		}


		return $norm;
	}
}
