<?php
/*
 *  $Id: recover.php 431 2011-12-03 18:11:04Z k42b3.x@googlemail.com $
 *
 * amun
 * A social content managment system based on the psx framework. For
 * the current version and informations visit <http://amun.phpsx.org>
 *
 * Copyright (c) 2010 Christoph Kappestein <k42b3.x@gmail.com>
 *
 * This file is part of amun. amun is free software: you can
 * redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or any later version.
 *
 * amun is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with amun. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * recover
 *
 * @author     Christoph Kappestein <k42b3.x@gmail.com>
 * @license    http://www.gnu.org/licenses/gpl.html GPLv3
 * @link       http://amun.phpsx.org
 * @category   module
 * @package    my
 * @version    $Revision: 431 $
 */
class recover extends Amun_ApplicationAbstract
{
	public function onLoad()
	{
		// add path
		$this->path->add('Login', $this->page->url . '/login');
		$this->path->add('Recover', $this->page->url . '/login/recover');


		// captcha
		$captcha = $this->config['psx_url'] . '/' . $this->config['psx_dispatch'] . 'api/system/captcha';

		$this->template->assign('captcha', $captcha);


		// template
		$this->htmlCss->add('my');

		$this->template->set('application/' . $this->page->application . '/login/' . __CLASS__ . '.tpl');
	}

	public function onPost()
	{
		try
		{
			$email   = $this->post->email('string', array(new PSX_Filter_Length(3, 64), new PSX_Filter_Email()));
			$captcha = $this->post->captcha('string');


			// check captcha if anonymous
			$captchaProvider = Amun_Captcha::factory($this->config['amun_captcha']);

			if(!$captchaProvider->verify($captcha))
			{
				throw new Amun_Exception('Invalid captcha');
			}


			if(!$this->validate->hasError())
			{
				$account = Amun_Sql_Table_Registry::get('User_Account')
					->select()
					->setColumns(array('id', 'name', 'email'))
					->addCondition('status', '!=', Amun_User_Account::BANNED)
					->addCondition('status', '!=', Amun_User_Account::ANONYMOUS)
					->addCondition('status', '!=', Amun_User_Account::NOT_ACTIVATED)
					->addCondition('status', '!=', Amun_User_Account::RECOVER)
					->getRow(PSX_Sql::FETCH_OBJECT)

				if($account instanceof Amun_User_Account)
				{
					if(!empty($account->email))
					{
						$ip    = $_SERVER['REMOTE_ADDR'];
						$token = Amun_Security::generateToken();
						$link  = $this->page->url . '/login/resetPw?token=' . $token;
						$date  = new DateTime('NOW', $this->register['core.default_timezone']);

						$account->setStatus(Amun_User_Account::RECOVER);
						$account->setIp($ip);
						$account->setToken($token);

						$handler = new Amun_User_Account_Handler($this->config, $this->sql, $this->registry, $this->user);
						$handler->update($account);


						$vars = array(

							'account.name' => $account->name,
							'host.name'    => $this->config->getHost(),
							'recover.ip'   => $ip,
							'recover.link' => $this->page->url . '/login/resetPw?token=' . $token,
							'recover.date' => $date->format($this->registry['core.format_date']),

						);

						$msg = $this->registry['my.recover_mail'];

						foreach($vars as $key => $value)
						{
							$msg = str_replace('{' . $key . '}', $value, $msg);
						}


						mail($email, $this->config->getHost() . ' password recovery', $msg);


						$this->template->assign('success', true);
					}
					else
					{
						throw new Amun_Exception('No public email address is set for this account');
					}
				}
				else
				{
					throw new Amun_Exception('Account does not exist or has not a valid status');
				}
			}
			else
			{
				throw new Amun_Exception($this->validate->getLastError());
			}
		}
		catch(Exception $e)
		{
			$this->template->assign('error', $e->getMessage());
		}
	}
}

