<?php
/*
 *  $Id: dispatch.php 440 2011-12-04 19:01:15Z k42b3.x@googlemail.com $
 *
 * amun
 * A social content managment system based on the psx framework. For
 * the current version and informations visit <http://amun.phpsx.org>
 *
 * Copyright (c) 2010 Christoph Kappestein <k42b3.x@gmail.com>
 *
 * This file is part of amun. amun is free software: you can
 * redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or any later version.
 *
 * amun is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with amun. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * dispatch
 *
 * @author     Christoph Kappestein <k42b3.x@gmail.com>
 * @license    http://www.gnu.org/licenses/gpl.html GPLv3
 * @link       http://amun.phpsx.org
 * @category   module
 * @version    $Revision: 440 $
 */
class dispatch extends PSX_ModuleAbstract
{
	private $application;

	private $registry;
	private $user;
	private $page;
	private $nav;
	private $path;
	private $gadget;

	public function __construct(PSX_Config $config, PSX_Template $template, $basePath, array $uriFragments)
	{
		parent::__construct($config, $template, $basePath, $uriFragments);


		// set xrds location header
		header('X-XRDS-Location: ' . $this->config['psx_url'] . '/' . $this->config['psx_dispatch'] . 'api/meta/xrds');


		// init application
		$this->application = Amun_Base_Application::initInstance($config, 'amun_' . md5($this->config['psx_url']));

		// assiged used object
		$this->registry = $this->application->getRegistry();
		$this->user     = $this->application->getUser();
		$this->page     = $this->application->getPage();
		$this->nav      = $this->application->getNav();
		$this->path     = $this->application->getPath();
		$this->gadget   = $this->application->getGadget();

		// assign to template
		$this->template->assign('sql', $this->application->getSql());
		$this->template->assign('registry', $this->application->getRegistry());
		$this->template->assign('user', $this->application->getUser());
		$this->template->assign('page', $this->page);
		$this->template->assign('nav', $this->nav);
		$this->template->assign('path', $this->path);
		$this->template->assign('gadget', $this->gadget);
		$this->template->assign('htmlJs', $this->application->getHtmlJs());
		$this->template->assign('htmlCss', $this->application->getHtmlCss());
		$this->template->assign('htmlContent', $this->application->getHtmlContent());
	}

	public function onLoad()
	{
		if(!empty($this->page->id))
		{
			// load extra rights
			if($this->page->hasRight())
			{
				$this->page->loadExtraRights();
			}


			// load nav
			if($this->page->hasNav())
			{
				$this->nav->load();
			}


			// load path
			if($this->page->hasPath())
			{
				$this->path->load();
			}


			// load gadgets
			if($this->page->hasGadget())
			{
				$this->gadget->load($this->page->id);
			}


			// load application
			$content = $this->loadApplication();

			$this->template->assign('content', $content);


			// set costum template if any
			if(!empty($this->page->template))
			{
				$this->template->set($this->page->template);
			}
			else
			{
				$this->template->set('page.tpl');
			}
		}
		else
		{
			throw new Amun_Page_Exception('Invalid page');
		}
	}

	private function loadApplication()
	{
		$path = 'application/' . $this->page->application . $this->page->applicationPath;

		if($this->page->cache)
		{
			$key     = 'c_p_' . md5($this->page->applicationPath) . '.htm';
			$handler = new PSX_Cache_Handler_File();
			$cache   = new PSX_Cache($key, $handler, $this->config);

			if(($content = $cache->load($key, $this->page->expire)) === false)
			{
				$content = $this->executeApplication($path);

				$cache->write($content);
			}
			else
			{
				# we have a cached content
				$content.= "\n" . '<!-- cached -->';
			}
		}
		else
		{
			$content = $this->executeApplication($path);
		}

		return $content;
	}

	public function executeApplication($path)
	{
		$applicationTemplate = new PSX_Template($this->config);

		$applicationTemplate->assign('registry', $this->registry);
		$applicationTemplate->assign('user', $this->user);
		$applicationTemplate->assign('page', $this->page);

		list($path, $file, $class, $method, $uriFragments) = $this->parsePath($path);

		$this->application->getHtmlCss()->add('default');

		$handle = new $class($this->config, $applicationTemplate, $path, $uriFragments, $this->application);

		if($handle instanceof Amun_ApplicationAbstract)
		{
			ob_start();

			try
			{
				$handle->_ini();

				$content = ob_get_contents();

				if(empty($content))
				{
					if($applicationTemplate->load() === false)
					{
						throw new Amun_Exception('Could not load template');
					}


					if(!($content = $applicationTemplate->transform()))
					{
						throw new Amun_Exception('Error while transforming template');
					}
				}
			}
			catch(Exception $e)
			{
				$content = '<p>' . $e->getMessage() . '</p>';
			}

			ob_end_clean();

			return $content;
		}
		else
		{
			throw new PSX_Loader_Exception('Class it is not an instance of Amun_ApplicationAbstract');
		}
	}

	public function parsePath($x, $deep = 0)
	{
		$x   = trim($x, '/');
		$fpc = PSX_Loader::getFPC($x);

		if($fpc !== false)
		{
			list($file, $path, $class) = $fpc;

			require_once($file);

			if(!class_exists($class, false))
			{
				throw new Amun_Exception('Class ' . $class . ' doesnt exist in ' . $file);
			}


			// control whether the method exists or not
			$method       = false;
			$rest         = substr($x, strlen($path) + 1);
			$classMethods = get_class_methods($class);
			$reserved     = array('onLoad', 'onPost', 'onPut', 'onDelete', 'onGet');

			if(!empty($rest))
			{
				$method = PSX_Loader::getPart($rest);

				if(in_array($method, $classMethods) && !in_array($method, $reserved))
				{
					$rest = substr($rest, strlen($method) + 1);
				}
				else
				{
					$method = false;
				}
			}


			// if we have no method look for an index
			if($method === false)
			{
				if(in_array('index', $classMethods))
				{
					$method = 'index';
				}
			}


			// get uri fragments
			$uriFragments = array();

			if(!empty($rest))
			{
				$uriFragments = explode('/', trim($rest, '/'));
			}
		}
		else
		{
			throw new Amun_Exception('Unkown request!');
		}

		return array(

			$path,
			$file,
			$class,
			$method,
			$uriFragments,

		);
	}
}

