<?php
/*
 *  $Id: register.php 442 2011-12-04 19:06:59Z k42b3.x@googlemail.com $
 *
 * amun
 * A social content managment system based on the psx framework. For
 * the current version and informations visit <http://amun.phpsx.org>
 *
 * Copyright (c) 2010 Christoph Kappestein <k42b3.x@gmail.com>
 *
 * This file is part of amun. amun is free software: you can
 * redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or any later version.
 *
 * amun is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with amun. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * register
 *
 * @author     Christoph Kappestein <k42b3.x@gmail.com>
 * @license    http://www.gnu.org/licenses/gpl.html GPLv3
 * @link       http://amun.phpsx.org
 * @category   module
 * @package    my
 * @version    $Revision: 442 $
 */
class register extends Amun_ApplicationAbstract
{
	public function onLoad()
	{
		if($this->user->hasRight('service_my_view'))
		{
			// gender
			$this->template->assign('gender', Amun_User_Account::getGender());


			// country
			$country = array();
			$result  = $this->sql->getAll('SELECT id, title FROM ' . $this->registry['table.system_country'] . ' ORDER BY title ASC');

			foreach($result as $row)
			{
				$country[$row['id']] = $row['title'];
			}

			$this->template->assign('country', $country);


			// captcha
			$captcha = $this->config['psx_url'] . '/' . $this->config['psx_dispatch'] . 'api/system/captcha';

			$this->template->assign('captcha', $captcha);


			// add path
			$this->path->add('Register', $this->page->url . '/register');


			// template
			$this->htmlCss->add('my');
			$this->htmlJs->add('jquery');

			$this->template->set('application/' . $this->page->application . '/' . __CLASS__ . '.tpl');
		}
		else
		{
			throw new Amun_Exception('Access not allowed');
		}
	}

	public function onPost()
	{
		try
		{
			$name      = $this->post->name('string', array(new PSX_Filter_Length(3, 32)), 'name', 'Name');
			$identity  = $this->post->identity('string', array(new PSX_Filter_Length(3, 128), new PSX_Filter_Email()), 'email', 'Email');
			$pw        = $this->post->pw('string');
			$pwRepeat  = $this->post->pwRepeat('string');
			$longitude = $this->post->longitude('float');
			$latitude  = $this->post->latitude('float');
			$captcha   = $this->post->captcha('string');

			if(!$this->validate->hasError())
			{
				// check whether registration is enabled
				if(!$this->registry['my.registration_enabled'])
				{
					throw new Amun_Exception('Registration is disabled');
				}


				// compare pws
				if(strcmp($pw, $pwRepeat) != 0)
				{
					throw new Amun_Exception('Password ist not the same');
				}


				// check captcha if anonymous
				$captchaProvider = Amun_Captcha::factory($this->config['amun_captcha']);

				if(!$captchaProvider->verify($captcha))
				{
					throw new Amun_Exception('Invalid captcha');
				}


				$account = new Amun_User_Account($this->config, $this->sql, $this->registry);
				$account->setGroupId($this->registry['core.default_user_group']);
				$account->setStatus(Amun_User_Account::NOT_ACTIVATED);
				$account->setIdentity($identity);
				$account->setName($name);
				$account->setPw($pw);
				$account->setLongitude($longitude);
				$account->setLatitude($latitude);


				$handler = new Amun_User_Account_Handler($this->config, $this->sql, $this->registry, $this->user);

				$account = $handler->create($account);


				if(isset($account->id))
				{
					// send activation mail
					$date = new DateTime('NOW', $this->register['core.default_timezone']);

					$vars = array(

						'account.name'     => $account->name,
						'account.identity' => $identity,
						'host.name'        => $this->config->getHost(),
						'register.link'    => $this->page->url . '/register/activate?token=' . $account->token,
						'register.date'    => $date->format($this->registry['core.format_date']),

					);

					$msg = $this->registry['my.register_mail'];

					foreach($vars as $key => $value)
					{
						$msg = str_replace('{' . $key . '}', $value, $msg);
					}


					mail($identity, $this->config->getHost() . ' activation', $msg);


					$this->template->assign('success', true);
				}
				else
				{
					throw new Amun_Exception('Your account was add for approval');
				}
			}
			else
			{
				throw new Amun_Exception($this->validate->getLastError());
			}
		}
		catch(Exception $e)
		{
			$this->template->assign('name', htmlspecialchars($name));
			$this->template->assign('identity', htmlspecialchars($identity));

			$this->template->assign('error', $e->getMessage());
		}
	}
}

