/*
 *  $Id: amun.js 422 2011-11-20 00:51:04Z k42b3.x@googlemail.com $
 *
 * amun
 * A social content managment system based on the psx framework. For
 * the current version and informations visit <http://amun.phpsx.org>
 *
 * Copyright (c) 2010 Christoph Kappestein <k42b3.x@gmail.com>
 *
 * This file is part of amun. amun is free software: you can
 * redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or any later version.
 *
 * amun is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with amun. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * amun
 *
 * @author     Christoph Kappestein <k42b3.x@gmail.com>
 * @license    http://www.gnu.org/licenses/gpl.html GPLv3
 * @link       http://amun.phpsx.org
 * @category   js
 * @version    $Revision: 422 $
 */
(function(win){

	var amun = {};

	amun.store = {};
	amun.store.instance = {};
	amun.store.editors = {};

	amun.client = function(f_id){

		var self = this;
		var method = 'POST';
		var override_method = 'POST';
		var target;
		var content_type = 'application/json';
		var data = {};

		var success_callback;
		var error_callback;
		var before_submit_callback;
		var after_submit_callback;


		this.get_data = function(){

			return data;

		}

		this.set_data = function(new_data){

			data = new_data;

		}

		this.get_content_type = function(){

			return content_type;

		}

		this.set_content_type = function(new_content_type){

			content_type = new_content_type

		}

		this.on_success = function(callback){

			success_callback = callback;

		}

		this.on_error = function(callback){

			error_callback = callback;

		}

		this.before_submit = function(callback){

			before_submit_callback = callback;

		}

		this.after_submit = function(callback){

			after_submit_callback = callback;

		}

		this.request = function(url, request_method, fields){

			// set data
			this.set_data(fields);


			// call before submit
			if(before_submit_callback)
			{
				before_submit_callback.call(self);
			}


			// get the request method
			switch(request_method)
			{
				case 'GET':

					method = 'GET';
					override_method = 'GET';

					break;

				case 'POST':

					method = 'POST';
					override_method = 'POST';

					break;

				case 'PUT':

					method = 'POST';
					override_method = 'PUT';

					break;

				case 'DELETE':

					method = 'POST';
					override_method = 'DELETE';

					break;
			}


			// construct the ajax request
			$.ajax({

				type: method,
				url: url,
				contentType: content_type,
				data: JSON.stringify(data),
				dataType: 'json',
				beforeSend: function(xhr){

					xhr.setRequestHeader('X-Http-Method-Override', override_method);
					xhr.setRequestHeader('Accept', 'application/json');

				},
				error: function(xhr, status, e){

					var message = JSON.parse(xhr.responseText);
					var text;

					if(typeof(message.text) != 'undefined')
					{
						text = message.text;
					}
					else if(status != null)
					{
						text = status;
					}
					else
					{
						text = 'An unknown error occured';
					}

					if(error_callback)
					{

						error_callback.call(self, text);
					}

					if(after_submit_callback)
					{
						after_submit_callback.call(self);
					}

				},
				success: function(data, status, xhr){

					if(data.success)
					{
						if(success_callback)
						{
							success_callback.call(self, data.text);
						}
					}
					else
					{
						if(error_callback)
						{
							error_callback.call(self, data.text);
						}
					}

					if(after_submit_callback)
					{
						after_submit_callback.call(self);
					}

				}

			});

		}


		$('#' + f_id).submit(function(){

			// get target
			var url = $(this).attr('action');


			// get request method
			var method = $(this).attr('method').toUpperCase();


			// get all form fields
			var arr = $(this).serializeArray();
			var fields = amun.store.instance;

			for(var i = 0; i < arr.length; i++)
			{
				fields[arr[i].name] = arr[i].value;
			}


			// if we have an ace editor
			if(typeof amun.store.editors != 'undefined')
			{
				for(var k in amun.store.editors)
				{
					var editor = amun.store.editors[k];
					var v = editor.getSession().getValue();

					fields[k] = v;
				}
			}


			// call request
			self.request(url, method, fields);


			return false;

		});

	}

	amun.form = function(c_id, url){

		var self = this;
		var container_id;

		var action;
		var method;
		var id;

		var load_callback;
		var error_callback;


		this.on_load = function(callback){

			load_callback = callback;

		}

		this.on_error = function(callback){

			error_callback = callback;

		}

		this.transform = function(form){

			$('#' + container_id).html(self.parse_elements(form));

			if(load_callback)
			{
				load_callback.call(self, form.ref);
			}

		}

		this.parse_elements = function(item){

			var html = '';

			switch(item['class'])
			{
				case 'form':

					html+= '<form action="' + item.action + '" method="' + item.method + '" id="' + item.ref + '">';

					for(var i = 0; i < item.item.children.item.length; i++)
					{
						html+= this.parse_elements(item.item.children.item[i]);
					}

					html+= '<p>';
					html+= '<input type="submit" value="Submit" />';
					html+= '</p>';

					html+= '</form>';

					break;

				case 'panel':

					html+= '<fieldset>';
					html+= '<legend>' + item.label + '</legend>';

					for(var i = 0; i < item.children.item.length; i++)
					{
						html+= this.parse_elements(item.children.item[i]);
					}

					html+= '</fieldset>';

					break;

				case 'captcha':

					var val = item.value ? item.value : '';

					html+= '<p>';
					html+= '<label for="' + item.ref + '">' + item.label + '</label><br />';
					html+= '<img src="' + item.src + '" alt="Captcha" />';
					html+= '<input type="' + item.label + '" name="' + item.ref + '" id="' + itemel.ref + '" value="' + val + '" />';
					html+= '</p>';

					break;

				case 'datalist':

					break;

				case 'input':

					var val = item.value ? item.value : '';
					var disabled = item.disabled ? 'disabled="disabled"' : '';

					switch(item.type)
					{
						case 'hidden':

							html+= '<input type="' + item.type + '" name="' + item.ref + '" id="' + item.ref + '" value="' + val + '" ' + disabled + ' />';

							break;

						default:

							html+= '<p>';
							html+= '<label for="' + item.ref + '">' + item.label + '</label><br />';
							html+= '<input type="' + item.type + '" name="' + item.ref + '" id="' + item.ref + '" value="' + val + '" ' + disabled + ' />';
							html+= '</p>';

							break;
					}

					break;

				case 'select':

					var val = item.value ? item.value : '';
					var disabled = item.disabled ? 'disabled="disabled"' : '';

					html+= '<p>';
					html+= '<label for="' + item.ref + '">' + item.label + '</label><br />';
					html+= '<select name="' + item.ref + '" id="' + item.ref + '" ' + disabled + '>';

					for(var j = 0; j < item.children.item.length; j++)
					{
						var opt = item.children.item[j];

						if(val == opt.value)
						{
							html+= '<option value="' + opt.value + '" selected="selected">' + opt.label + '</option>';
						}
						else
						{
							html+= '<option value="' + opt.value + '">' + opt.label + '</option>';
						}
					}

					html+= '</select>';
					html+= '</p>';

					break;

				case 'textarea':

					var val = item.value ? item.value : '';
					var disabled = item.disabled ? 'disabled="disabled"' : '';

					html+= '<p>';
					html+= '<label for="' + item.ref + '">' + item.label + '</label><br />';
					html+= '<textarea name="' + item.ref + '" id="' + item.ref + '" ' + disabled + '>' + val + '</textarea>';
					html+= '</p>';

					break;
			}

			return html;
		}


		container_id = c_id;

		$.ajax({

			type: 'GET',
			url: url,
			dataType: 'json',
			beforeSend: function(xhr){

				xhr.setRequestHeader('Accept', 'application/json');

			},
			error: function(xhr, status, e){

				if(error_callback)
				{
					error_callback.call(self, e);
				}

			},
			success: function(data, status, xhr){

				if(typeof data.success != 'undefined' && !data.success)
				{
					if(error_callback)
					{
						error_callback.call(self, data.text);
					}
				}
				else
				{
					self.transform(data);
				}

			}

		});

	}

	win.amun = amun;

})(window);





